import { EndPoints, getData } from "../services/AccessAPI";
import { useNavigate } from "react-router-dom";
import { useEffect } from "react";
import { useState } from 'react';
import { useLocation } from 'react-router-dom';
import { Routes } from "../services/Routes";
import { Collapse } from "reactstrap";
import RowButton from "../ui-elements/table/RowButton";
import DataFormatter, { RenderType } from "../ui-elements/table/DataFormatter";
import TranactionTable from "../ui-elements/table/TransactionTable";
import EditButton from "../ui-elements/controls/buttons/EditButton";
import DeleteButton from "../ui-elements/controls/buttons/DeleteButton";
import ExecuteButton from "../ui-elements/controls/buttons/ExecuteButton";
import AddButton from "../ui-elements/controls/buttons/AddButton";

export default function AccountDashboard() {
    const search = useLocation().search;
    const id = new URLSearchParams(search).get('id')

    const [page, setPage] = useState(1);

    useEffect(() => {
        getData(EndPoints.ACCOUNTS + "/" + id).then(
            (result) => {
                if (result) {
                    setAccount({
                        name: result.name,
                        lastActivity: result.lastActivity,
                        createdOn: result.createdOn,
                        balance: result.balance,
                        initialBalance: result.initialBalance,
                        virtualBalance: result.virtualBalance,
                        currencyId: result.currencyId,
                        externalAccountNumber: result.externalAccountNumber,
                        loading: false
                    });
                }
            }
        );

        getAllTransactionsData(page);
        getAllAccountsData();
        getAccountEnvelopeData();
        getAccountAutoclassRulesData(id);
        getAccountHistorical(id, 12);
    }, [id])

    let navigate = useNavigate();

    const [isAutoclassOpen, setIsAutoclassOpen] = useState(false);
    const [loading, setLoading] = useState(true);

    const [transactions, setTransactions] = useState([]);
    const [accounts, setAccounts] = useState([]);
    const [envelopes, setEnvelopes] = useState([]);
    const [autoclassRules, setAutoclassRules] = useState([]);
    const [accountHistorical, setAccountHistorical] = useState({});

    const [autoclassRuleTriggerMessage, setAutoclassRuleTriggerMessage] = useState("");
    
    const [account, setAccount] = useState({
        name: '',
        lastActivity: null,
        createdOn: null,
        balance: 0,
        initialBalance: 0,
        virtualBalance: 0,
        currencyId: 0,
        externalAccountNumber: '',
        loading: true
    });

    //const [users, setUsers] = useState(null);

    function onTransactionEdit(id){
        let query = "?id=" + id
        navigate(Routes.TRANSACTIONS_EDIT + query);
    }

    function onTransactionDelete(id){
        let query = "?id=" + id
        navigate(Routes.TRANSACTIONS_DELETE + query);
    }

    function previousPage()
    {
        setLoading(true);
        setPage(page > 1 ? page - 1 : 1)
        getAllTransactionsData(page)
    }

    function nextPage()
    {
        setLoading(true);
        setPage(page + 1);
        getAllTransactionsData(page)
    }

    function getAllTransactionsData(dataPage) {
        let pageSize = 50;
        let query = "?PageNumber=" + dataPage + "&PageSize=" + pageSize + "&accountId=" + id
        getData(EndPoints.TRANSACTIONS + query).then(
            (result) => {
                if (result) {
                    console.log()
                    setTransactions(result);
                    setLoading(false);
                }
            }
        );
    }

    /*function getAllUsersData() {
        getData(EndPoints.USERS).then(
            (result) => {
                if (result) {
                    setUsers(result);
                }
            }
        );
    }*/

    function getAllAccountsData() {
        getData(EndPoints.ACCOUNTS).then(
            (result) => {
                if (result) {
                    setAccounts(result);
                    setLoading(false);
                }
            }
        );
    }

    function getAccountEnvelopeData() {
        getData(EndPoints.ENVELOPES).then(
            (result) => {
                if (result) {
                    setEnvelopes(result);
                    setLoading(false);
                }
            }
        );
    }

    function getAccountAutoclassRulesData(id) {
        getData(EndPoints.AUTOCLASS + "?account=" + id).then(
            (result) => {
                if (result) {
                    setAutoclassRules(result);
                    setLoading(false);
                }
            }
        );
    }

    function getAccountHistorical(id, lookBack) {
        getData(EndPoints.ACCOUNTS_HISTORICAL + "/" + id + "?lookBack=" + lookBack).then(
            (result) => {
                if (result) {
                    setAccountHistorical(result);
                    console.log("Account Historical: ");
                    console.log(accountHistorical);
                    setLoading(false);
                }
            }
        );
    }

    function triggerAutoclassRule(id) {
        getData(EndPoints.AUTOCLASS_TRIGGERAUTOCLASSRULE + "/" + id).then(
            (result) => {
                if (result) {
                    setAutoclassRuleTriggerMessage(result.message);
                    setLoading(false);
                }
            }
        );
    }

    function findAccountName(id) {
        let name = accounts?.find(x => x.id === id)?.name;
        console.log("Id: " + id + ", Name: " + name);
        return name;
    }

    function findEnvelopeName(id) {
        let name = envelopes?.find(x => x.id === id)?.name;
        console.log("Id: " + id + ", Name: " + name);
        return name;
    }

    function renderTransactionsTable() {
        return (
            <TranactionTable 
                transactions={transactions} 
                setTransactions={setTransactions}
                referenceAccountId={id}
                signedAmount={true}
            />
        );
    }

    function renderEnvelopesTable() {
        return (
            <table className="table table-sm table-striped">
                <thead>
                    <tr>
                        <th>Name</th>
                        <th>Balance</th>
                        <th>Status</th>
                        <th>Persistant</th>
                        <th>Priority</th>
                        <th>Action</th>
                    </tr>
                </thead>
                <tbody>
                    {
                        envelopes.map(env => (
                            <tr key={env.id}>
                                <td><DataFormatter data={env.name} /></td>
                                <td><DataFormatter data={env.balance} renderType={RenderType.MoneyUnsigned}/></td>
                                <td><DataFormatter data={env.enabled ? "Enabled" : "Disabled"} /></td>
                                <td><DataFormatter data={env.isPersistant ? "True" : "False"} /></td>
                                <td><DataFormatter data={env.priority} /></td>
                                <td>
                                    <EditButton
                                        className="btn btn-sm btn-primary"
                                        onClick={() => navigate(Routes.ENVELOPES_EDIT + "?id=" + env.id)}
                                    />
                                    &nbsp;
                                    <DeleteButton
                                        className="btn btn-sm btn-danger"
                                        onClick={() => navigate(Routes.ENVELOPES_DELETE + "?id=" + env.id)}
                                    />
                                </td>
                            </tr>
                        ))
                    }
                </tbody>
            </table>
        );
    }

    function getRuleColor(enabled) {
        return enabled ? "#abfcae" : "fcabad";
    }

    function renderAutoclassTable() {
        return (
            <div className="row">
                <div className="container-fluid">
                    {
                        autoclassRules.map(rule => (
                            <>
                                <div className="row" style={{backgroundColor: getRuleColor(rule.enabled)}}>
                                    <div className="col-6">
                                        <strong>{rule.name}</strong>
                                    </div>
                                    <div className="col-6">
                                        {rule.enabled ? "Enabled" : "Disabled"}
                                        &nbsp;
                                        &nbsp;
                                        <EditButton
                                            className="btn btn-sm btn-primary"
                                            onClick={() => navigate(Routes.AUTOCLASS_EDIT + "?id=" + rule.id)}
                                        />
                                        &nbsp;
                                        <DeleteButton
                                            className="btn btn-sm btn-danger"
                                            onClick={() => navigate(Routes.AUTOCLASS_DELETE + "?id=" + rule.id)}
                                        />
                                        &nbsp;
                                        <ExecuteButton
                                            className="btn btn-sm btn-warning"
                                            onClick={() => triggerAutoclassRule(rule.id)}
                                        />

                                        <span>{autoclassRuleTriggerMessage}</span>
                                    </div>
                                </div>
                                <div className="row">
                                    <div className="col-6 text-center">If...</div>
                                    <div className="col-6 text-center">...then:</div>
                                </div>
                                <div className="row">
                                    <div className="col-6 text-center">
                                        {
                                            rule.expressions.map(exp => (
                                                <>
                                                    <div className="row">
                                                        <div className="col-12 text-center">
                                                            {exp.transactionField} is {exp.compareOperator} {exp.value}
                                                        </div>
                                                    </div>
                                                </>
                                            ))
                                        }
                                    </div>
                                    <div className="col-6 text-center">
                                        {
                                            rule.changes.map(chg => (
                                                <>
                                                    <div className="row">
                                                        <div className="col-12 text-center">
                                                            {chg.field} is set to "{chg.value}"
                                                        </div>
                                                    </div>
                                                </>
                                            ))
                                        }
                                    </div>
                                </div>
                            </>
                        ))
                    }
                </div>
            </div>
        );
    }

    function medianIncome()
    {
        if (accountHistorical.income === null || accountHistorical.income === undefined )
            return 0;

        return accountHistorical.income.median;
    }

    let transactionsContent = loading ? (
        <p>
            <em>Loading...</em>
        </p>
    ) : (renderTransactionsTable())

    let envelopesContent = loading ? (
        <p>
            <em>Loading...</em>
        </p>
    ) : (renderEnvelopesTable())

    let autoclassContent = loading ? (
        <p>
            <em>Loading...</em>
        </p>
    ) : (renderAutoclassTable())

    const toggleAutoClassDialog = () => setIsAutoclassOpen(!isAutoclassOpen);
    //<DataFormatter data={transaction.date} renderType={RenderType.Date} />
    return (
        <div>
            <h2>{account.name}
                &nbsp;
                <EditButton
                    className="btn btn-sm btn-primary"
                    onClick={() => navigate(Routes.ACCOUNTS_EDIT + "?id=" + id)}
                />
                &nbsp;
                <DeleteButton
                    className="btn btn-sm btn-danger"
                    onClick={() => navigate(Routes.ACCOUNTS_DELETE + "?id=" + id)}
                />
            </h2>
            
            <div className="container-fluid">
                <div className="row">
                    <div className="col-md-6">
                        <h3>Balance : <DataFormatter data={account.balance} renderType={RenderType.MoneyUnsigned} /></h3>
                        <h4>Virtual Balance : <DataFormatter data={account.virtualBalance} renderType={RenderType.MoneyUnsigned} /> / <DataFormatter data={medianIncome()} renderType={RenderType.MoneyUnsigned} /></h4>
                        <h5>Last activity: <DataFormatter data={account.lastActivity} renderType={RenderType.Date} /></h5>
                    </div>
                    <div className="col-md-6">
                        <h4><DataFormatter data={account.externalAccountNumber} /></h4>
                    </div>
                </div>
                <hr />
                <div className="row">
                    <div className="text-center">
                        <h5 className="">
                            {
                                autoclassRules?.length > 0 ?
                                    <button onClick={toggleAutoClassDialog} className="btn btn-sm btn-outline-primary">{isAutoclassOpen ? "Hide " + autoclassRules?.length + " Autoclass Rules" : "Show " + autoclassRules?.length + " Autoclass Rules"}</button>
                                    : null
                            }
                            
                            &nbsp;
                            
                            <button onClick={() => navigate(Routes.AUTOCLASS_CREATE)} className="btn btn-sm btn-outline-primary">Create Autoclass Rule</button>
                        </h5>
                    </div>
                </div>
                <div className="row">
                    <Collapse isOpen={isAutoclassOpen}>
                        {autoclassContent}
                    </Collapse>
                </div>
                <div className="row">
                    <div className="col-md-12">
                        <div className="row">
                            <div className="text-center">
                                <h3 className="">
                                    Envelopes
                                    &nbsp;
                                    <AddButton
                                        className="btn btn-sm btn-primary"
                                        onClick={() => navigate(Routes.ENVELOPES_CREATE + "?id=" + id)}
                                    />
                                </h3>
                            </div>
                        </div>
                        {envelopesContent}
                    </div>
                </div>
                <div className="row">
                    <div className="col-md-12">
                        <div className="row">
                            <div className="text-center">
                                <h3 className="">
                                    Transactions
                                    &nbsp;
                                    <AddButton
                                        className="btn btn-sm btn-primary"
                                        onClick={() => navigate(Routes.TRANSACTIONS_CREATE)}
                                    />
                                </h3>
                            </div>
                        </div>
                        <div className="row">
                            <div className="col-md-4">
                            </div>
                            <div className="col-md-1">
                                <button onClick={() => previousPage()} className="btn btn-primary">Previous Page</button>
                            </div>
                            <div className="col-md-2">

                            <div className="text-center">
                                <h3>
                                    {page}
                                </h3>
                            </div>
                            </div>
                            <div className="col-md-1">
                                <button onClick={() => nextPage()} className="btn btn-primary">Next Page</button>
                            </div>
                            <div className="col-md-4">
                            </div>
                        </div>
                        {transactionsContent}
                    </div>
                </div>
            </div>
        </div>
    );
}