import { EndPoints, getData } from "../services/AccessAPI";
import { useNavigate } from "react-router-dom";
import { useEffect } from "react";
import { useState } from 'react';
import { Routes } from "../services/Routes";
import RowButton from "../ui-elements/table/RowButton";
import DataFormatter, { RenderType } from "../ui-elements/table/DataFormatter";
import EditButton from "../ui-elements/controls/buttons/EditButton";
import DeleteButton from "../ui-elements/controls/buttons/DeleteButton";
import RefreshButton from "../ui-elements/controls/buttons/RefreshButton";

export default function Accounts() {
    useEffect(() => {
        getAllAccountsData();
        getAllUsersData();
    }, [])

    let navigate = useNavigate();

    const [state, setState] = useState({
        accounts: [],
        loading: false
    });

    const [users, setUsers] = useState(null);

    function onAccountCreate() {
        navigate(Routes.ACCOUNTS_CREATE);
    }

    function onAccountDashboard(id) {
        let query = "?id=" + id
        navigate(Routes.ACCOUNTS_DASHBOARD + query);
    }

    function onAccountEdit(id) {
        let query = "?id=" + id
        navigate(Routes.ACCOUNTS_EDIT + query);
    }

    function onAccountDelete(id) {
        let query = "?id=" + id
        navigate(Routes.ACCOUNTS_DELETE + query);
    }

    function getAllAccountsData() {
        getData(EndPoints.ACCOUNTS).then(
            (result) => {
                if (result) {
                    setState({
                        accounts: result,
                        loading: false
                    });
                }
            }
        );
    }

    function getAllUsersData() {
        getData(EndPoints.USERS).then(
            (result) => {
                if (result) {
                    setUsers(result);
                }
            }
        );
    }

    function refreshAccount(id) {
        getData(EndPoints.ACCOUNTS_REFRESHBALANCE + "/" + id).then(
            (result) => {
                if (result) {
                    getAllAccountsData();
                }
            }
        );
    }

    function findUserName(id) {
        return users?.find(x => x.id)?.username;
    }

    function renderAllAccountsTable(accounts) {
        return (
            <div className="container-fluid">
                <div className="row">
                    <div className="col-md-12">
                        <table className="table table-sm table-striped">
                            <thead>
                                <tr>
                                    <th>Name</th>
                                    <th>Owner</th>
                                    <th>Balance</th>
                                    <th>External Account Number</th>
                                    <th>Actions</th>
                                </tr>
                            </thead>
                            <tbody>
                                {
                                    accounts.map(account => (
                                        <tr key={account.id}>
                                            <td><button className="btn btn-sm btn-clear" onClick={() => onAccountDashboard(account.id)}><DataFormatter data={account.name} /></button></td>
                                            <td><DataFormatter data={findUserName(account.ownerId)} /></td>
                                            <td><DataFormatter data={account.balance} renderType={RenderType.MoneyUnsigned} /></td>
                                            <td><DataFormatter data={account.externalAccountNumber} /></td>
                                            <td>
                                                <EditButton
                                                    className="btn btn-sm btn-primary"
                                                    onClick={() => onAccountEdit(account.id)}
                                                />
                                                &nbsp;
                                                <DeleteButton
                                                    className="btn btn-sm btn-danger"
                                                    onClick={() => onAccountDelete(account.id)}
                                                />
                                                &nbsp;
                                                <RefreshButton
                                                    className="btn btn-sm btn-warning"
                                                    onClick={() => refreshAccount(account.id)}
                                                />
                                            </td>
                                        </tr>
                                    ))
                                }
                            </tbody>
                        </table>
                    </div>
                </div>
            </div>
        );
    }

    let content = state.loading ? (
        <p>
            <em>Loading...</em>
        </p>
    ) : (
        renderAllAccountsTable(state.accounts)
    )

    
    return (
        <div>
            <h3>List of Accounts</h3>
            <button onClick={() => onAccountCreate()} className="btn btn-primary">Create new account</button>
            {content}
        </div>
    );
}