import SessionManager from "../auth/SessionManager";
import { EndPoints, postData } from "../services/AccessAPI";
import { useNavigate } from "react-router-dom";
import { useState } from 'react';
import { Routes } from "../services/Routes";
import TextInput from "../ui-elements/controls/input/TextInput";

export default function CreateAccount() {
    let navigate = useNavigate();

    const [name, setName] = useState("");
    //const [owner, setOwner] = useState(0);
    const [initBalance, setInitBalance] = useState("");
    //const [currency, setCurrency] = useState(0);
    const [extNum, setExtNum] = useState("");
    const [loading, setLoading] = useState(true);

    function onSubmit(e) {
        e.preventDefault();

        let accountObj = {
            name: name,
            owner: SessionManager.getUserId(),
            initialBalance: initBalance,
            currency: 1,
            externalAccountNumber: extNum,
        }

        postData(EndPoints.ACCOUNTS, accountObj).then((result) => {
            setLoading(false);
            let responseJson = result;
            if (responseJson) {
                navigate(Routes.ACCOUNTS);
            }
        });
    }

    function onClickBack(e){
        e.preventDefault();

        if(SessionManager.getToken()){
            navigate(Routes.ACCOUNTS);
        }else{
            navigate(Routes.LOGIN);
        }   
    }

    return (
        <div className="row">
            <div className="col-md-4">
                <h3>Create new account</h3>
                <form onSubmit={onSubmit}>
                    <div className="form-group">
                        <label className="control-label">Account Name: </label>
                        <TextInput
                            className="form-control"
                            value={name}
                            onChange={(e) => setName(e.target.value)}
                            name="name"
                        />
                    </div>

                    <br />
                    <div className="form-group">
                        <label className="control-label">Initial Balance: </label>
                        <TextInput
                            className="form-control"
                            value={initBalance}
                            onChange={(e) => setInitBalance(e.target.value)}
                            name="initBalance"
                        />
                    </div>

                    <br />
                    <div className="form-group">
                        <label className="control-label">External Account Number: </label>
                        <TextInput
                            className="form-control"
                            value={extNum}
                            onChange={(e) => setExtNum(e.target.value)}
                            name="extNum"
                        />
                    </div>

                    <br />
                    <div className="form-group">
                        <input type="button" value="Cancel" onClick={onClickBack} className="btn btn-primary"></input>
                        &nbsp;&nbsp;
                        <input type="submit" value="Create Account" className="btn btn-success"></input> 
                    </div>
                </form>
            </div>
        </div>
    );
}