import { EndPoints, getData } from "../services/AccessAPI";
import { useNavigate } from "react-router-dom";
import { useEffect } from "react";
import { useState } from 'react';
import { Routes } from "../services/Routes";
import DataFormatter, { RenderType } from "../ui-elements/table/DataFormatter";
import EditButton from "../ui-elements/controls/buttons/EditButton";
import DeleteButton from "../ui-elements/controls/buttons/DeleteButton";
import RefreshButton from "../ui-elements/controls/buttons/RefreshButton";

export default function ApiKeysPage() {
    useEffect(() => {
        getApiKeys();
    }, [])

    let navigate = useNavigate();

    const [apiKeys, setApiKeys] = useState([]);
    const [loading, setLoading] = useState(true);

    function getApiKeys() {
        getData(EndPoints.APIKEY).then(
            (result) => {
                if (result) {
                    console.log("APIKEY api call return object");
                    console.log(result);

                    setApiKeys(result.keys);
                    setLoading(false);
                }
            }
        );
    }

    function refreshApiKey(apiKey) {
        getData(EndPoints.APIKEY_CREATENEW).then(
            (result) => {
                if (result) {
                    getApiKeys();
                }
            }
        );
    }

    function onApiKeyDelete(apiKey) {
        getData(EndPoints.APIKEY + "/" + apiKey).then(
            (result) => {
                if (result) {
                    getApiKeys();
                }
            }
        );
    }

    function renderAllAccountsTable() {
        return (
            <div className="container-fluid">
                <div className="row">
                    <div className="col-md-12">
                        <table className="table table-sm table-striped">
                            <thead>
                                <tr>
                                    <th>API Key</th>
                                    <th>Actions</th>
                                </tr>
                            </thead>
                            <tbody>
                                {
                                    apiKeys.map(apikey => (
                                        <tr key={apikey}>
                                            <td><DataFormatter data={apikey} /></td>
                                            <td>
                                                {/*<DeleteButton
                                                    className="btn btn-sm btn-danger"
                                                    onClick={() => onApiKeyDelete(apikey)}
                                                />*/}
                                                &nbsp;
                                                <RefreshButton
                                                    className="btn btn-sm btn-warning"
                                                    onClick={() => refreshApiKey(apikey)}
                                                />
                                            </td>
                                        </tr>
                                    ))
                                }
                            </tbody>
                        </table>
                    </div>
                </div>
            </div>
        );
    }

    let content = loading ? (
        <p>
            <em>Loading...</em>
        </p>
    ) : (
        renderAllAccountsTable()
    )

    
    return (
        <div>
            <h3>API Key</h3>
            {content}
        </div>
    );
}