import { useNavigate } from "react-router-dom";
import { useEffect } from "react";
import { useState } from 'react';
import { EndPoints, postDataForLogin, getData } from "../services/AccessAPI";
import SessionManager from "./SessionManager";
import { Routes } from "../services/Routes";
import TextInput from "../ui-elements/controls/input/TextInput";

export default function Login() {

    useEffect(() => {
        getData(EndPoints.USER_CANREGISTER).then(
            (result) => {
                if (result) {
                    setCanRegister(result.allowed)
                }
            }
        );
    }, []);

    const [username, setUsername] = useState("");
    const [password, setPassword] = useState("");
    const [loading, setLoading] = useState(false);
    const [failed, setFailed] = useState(false);
    const [error, setError] = useState(false);
    const [canRegister, setCanRegister] = useState(false);

    let navigate = useNavigate();

    function onKeyDown(e) {
        if (e.key === 'Enter') {
            login();
        }
    }

    function login() {
        setLoading(true);

        let userObj = {
            username: username,
            password: password,
        }

        //console.log("login info: " + userInfo.password);
        postDataForLogin(EndPoints.USER_AUTHENTICATE, userObj).then((result) => {
            if (result?.token) {

                SessionManager.setUserSession(result.username, result.token, result.id, result.firstName, result.lastName)

                if (SessionManager.getToken()) {
                    setLoading(false);
                    navigate(Routes.HOME);
                }
            }

            else {
                let errors = '';
                for (const key in result?.errors) {
                    if (Object.hasOwnProperty.call(result.errors, key)) {
                        errors += result.errors[key];

                    }
                }
                errors = errors === '' ? 'Login is unsuccessfull!' : errors;
                /*toast.error(errors, {
                    position: "top-right",
                    autoClose: 5000,
                    hideProgressBar: true,
                    closeOnClick: true,
                    pauseOnHover: true,
                    draggable: true
                });*/

                setError("Login failed!")
                setLoading(false)
            }

        });
    }

    function registration() {
        navigate(Routes.REGISTER);
    }

    
    let content;
    if (loading) {
        content = <div>Loading...</div>;
    } else if (error !== '') {
        content = <div>ERROR: {error}</div>
    }

    return (
        <div className="row">
            <div className="login-box col-md-4">
                <div className="login-logo">
                    <a href="/"><b>ECommerce</b></a>
                </div>
                <div className="login-box-body">
                    <p className="login-box-msg">Sign in to access the application</p>

                    <div className="form-group has-feedback">

                        <TextInput
                            className="form-control"
                            placeholder="Username"
                            name="userName"
                            onChange={(e) => setUsername(e.target.value)}
                            onKeyDown={onKeyDown}
                        />

                        <span className="glyphicon glyphicon-user form-control-feedback" />
                    </div>
                    <br />
                    <div className="form-group has-feedback">
                        <TextInput
                            className="form-control"
                            placeholder="Password"
                            type="password"
                            name="password"
                            onChange={(e) => setPassword(e.target.value)}
                        />

                        <span className="glyphicon glyphicon-lock form-control-feedback" />
                    </div>
                    <br />
                    <div className="row">
                        <div className="col-md-4">
                            <button className="btn btn-primary btn-block" onClick={login}>
                                Sign In
                            </button>
                        </div>
                        <div className="col-md-6">
                            {canRegister && <button className="btn btn-primary btn-block" onClick={registration}>
                                Create an account
                            </button>}
                        </div>
                        <div className="col-md-2">
                            {content}
                        </div>
                    </div>
                    <div className="row">
                        <div className="col-md-8">
                            <strong className="has-error">{error}</strong>
                        </div>
                        <div className="col-md-4">
                            {/*<ToastContainer></ToastContainer>*/}
                        </div>
                    </div>
                </div>
            </div>
        </div>
    );
}