import { useNavigate } from "react-router-dom";
import { useEffect } from "react";
import { useState } from 'react';
import { EndPoints, postData } from "../services/AccessAPI";
import { Routes } from "../services/Routes";


export default function Register() {

    useEffect(() => {
    }, []);

    const [firstName, setFirstName] = useState("");
    const [lastName, setLastName] = useState("");
    const [email, setEmail] = useState("");
    const [username, setUsername] = useState("");
    const [password, setPassword] = useState("");
    const [confirmationPassword, setConfirmationPassword] = useState("");
    const [role, setRole] = useState("");
    const [loading, setLoading] = useState(false);
    const [error, setError] = useState(false);

    let navigate = useNavigate();

    function onKeyDown(e) {
        if (e.key === 'Enter') {
            register();
        }
    }

    function register() {
        setLoading(true)

        if (password !== confirmationPassword) {
            setError("Password and confirm password are not same");
            return;
        }

        let userObj = {
            firstName: firstName,
            lastName: lastName,
            email: email,
            username: username,
            password: password,
            role: role,
        }

        postData(EndPoints.USER_REGISTER, userObj).then((result) => {
            if (result) {
                navigate(Routes.LOGIN);
            } else {
                let errors = '';
                for (const key in result?.errors) {
                    if (Object.hasOwnProperty.call(result.errors, key)) {
                        errors += result.errors[key];

                    }
                }
                errors = errors === '' ? 'Registration was unsuccessfull!' : errors;
                /*toast.error(errors, {
                    position: "top-right",
                    autoClose: 5000,
                    hideProgressBar: true,
                    closeOnClick: true,
                    pauseOnHover: true,
                    draggable: true
                });*/

                setError("Registration failed!")
                setLoading(false)
            }
        });
    }

    
    let content;
    if (loading) {
        content = <div>Loading...</div>;
    } else if (error !== '') {
        content = <div>ERROR: {error}</div>
    } else {
        content = ""
    }

    return (
        <div className="row">
            <div className="login-box col-md-4">
                <div className="login-logo">
                    <a href="/"><b>ECommerce</b></a>
                </div>
                <div className="login-box-body">
                    <p className="login-box-msg">Register user account</p>
                    <div className="form-group has-feedback">
                        <input
                            type="text"
                            className="form-control"
                            placeholder="Please Enter First Name"
                            name="firstName"
                            onChange={(e) => setFirstName(e.target.value)}
                            onKeyDown={onKeyDown}
                        />
                        <span className="glyphicon glyphicon-user form-control-feedback" />
                    </div>
                    <div className="form-group has-feedback">
                        <input
                            type="text"
                            className="form-control"
                            placeholder="Please Enter Last Name"
                            name="lastName"
                            onChange={(e) => setLastName(e.target.value)}
                            onKeyDown={onKeyDown}
                        />
                        <span className="glyphicon glyphicon-user form-control-feedback" />
                    </div>
                    <div className="form-group has-feedback">
                        <input
                            type="text"
                            className="form-control"
                            placeholder="Please Enter Username"
                            name="userName"
                            onChange={(e) => setUsername(e.target.value)}
                            onKeyDown={onKeyDown}
                        />
                        <span className="glyphicon glyphicon-user form-control-feedback" />
                    </div>
                    <div className="form-group has-feedback">
                        <input
                            type="text"
                            className="form-control"
                            placeholder="Please Enter Email"
                            name="email"
                            onChange={(e) => setEmail(e.target.value)}
                            onKeyDown={onKeyDown}
                        />
                        <span className="glyphicon glyphicon-user form-control-feedback" />
                    </div>
                    <div className="form-group has-feedback">
                        <input
                            type="text"
                            className="form-control"
                            placeholder="Please Enter Role"
                            name="role"
                            onChange={(e) => setRole(e.target.value)}
                            onKeyDown={onKeyDown}
                        />
                        <span className="glyphicon glyphicon-user form-control-feedback" />
                    </div>
                    <div className="form-group has-feedback">
                        <input type="password" className="form-control" placeholder="Please Enter Password" name="password"
                            onChange={(e) => setPassword(e.target.value)} onKeyDown={onKeyDown}
                        />
                        <span className="glyphicon glyphicon-lock form-control-feedback" />
                    </div>
                    <div className="form-group has-feedback">
                        <input type="password" className="form-control" placeholder="Please Confirm Password" name="passwordConfirm"
                            onChange={(e) => setConfirmationPassword(e.target.value)} onKeyDown={onKeyDown}
                        />
                        <span className="glyphicon glyphicon-lock form-control-feedback" />
                    </div>
                    <div className="row">
                        <div className="col-md-4">
                            <button className="btn btn-primary btn-block" onClick={register}>
                                Register
                            </button>
                        </div>
                        <div className="col-md-2">
                            {content}
                        </div>
                    </div>
                    <div className="row">
                        <div className="col-md-8">
                            <strong className="has-error">{error}</strong>
                        </div>
                        <div className="col-md-4">
                            {/*<ToastContainer></ToastContainer>*/}
                        </div>
                    </div>
                </div>
            </div>
        </div>
    );
}