import React from 'react';
import { EndPoints, getData } from "../services/AccessAPI";
//import { useNavigate } from "react-router-dom";
import { useState } from 'react';
import { useEffect } from "react";
import DatePicker from 'react-datepicker';
import Select from 'react-select';
import RowButton from "../ui-elements/table/RowButton";
import { GetUniqueRenderKey } from '../helpers/DataValidation';
//import '../main.css';

export default function AutoclassChangeBuilder({ changes, setChangeState }) {
    useEffect(() => {
        getFieldInfoData();

        getAllAccountsDataAsSelectInput();
        getAllEnvelopesDataAsSelectInput();
        getAllCurrencyTypeDataAsSelectInput();
    }, [])

    const [accounts, setAccounts] = useState([]);
    const [envelopes, setEnvelopes] = useState([]);
    const [currencyTypes, setCurrencyTypes] = useState([]);
    const [fields, setFields] = useState([]);
    
    const [loading, setLoading] = useState(true);

    function getAllAccountsDataAsSelectInput() {
        getData(EndPoints.ACCOUNTS).then(
            (result) => {
                if (result) {
                    setAccounts(result.map(a => {
                        return {
                            value: a.id,
                            label: a.name
                        }
                    }));
                    setLoading(false);
                }
            }
        );
    }

    function getAllEnvelopesDataAsSelectInput() {
        getData(EndPoints.ENVELOPES).then(
            (result) => {
                if (result) {
                    setEnvelopes(result.map(e => {
                        return {
                            value: e.id,
                            label: e.name
                        }
                    }));
                    setLoading(false);
                }
            }
        );
    }


    function getAllCurrencyTypeDataAsSelectInput() {
        getData(EndPoints.CURRENCYTYPES).then(
            (result) => {
                if (result) {
                    setCurrencyTypes(result.map(c => {
                        return {
                            value: c.id,
                            label: c.code
                        }
                    }));
                    setLoading(false);
                }
            }
        );
    }

    function getFieldInfoData() {
        getData(EndPoints.AUTOCLASS_FIELDINFO).then(
            (result) => {
                if (result) {
                    setFields(result.map(a => {
                        return {
                            value: a,
                            label: a.name
                        }
                    }));

                    setLoading(false);
                }
            }
        );
    }

    function createChange() {
        const changeList = [...changes];

        let change = {
            id: GetUniqueRenderKey(),
            field: fields[0].value.name,
            type: fields[0].value.type,
            value: ""
        }

        changeList.push(change);
        setChangeState(changeList);
    }

    function setNewField(newField, change) {
        const changeList = [...changes];

        changeList.forEach(c => {
            if (c.id === change.id) {
                c.field = newField.name;
                c.type = newField.type;
                setNewValue(c.value, change)
            }
        });

        setChangeState(changeList);
    }

    function setNewValue(newValue, change) {
        const changeList = [...changes];

        changeList.forEach(c => {
            if (c.id === change.id) {
                switch (c.type) {
                    case "INT":
                        c.value = typeof newValue === 'string' ? newValue.replace(/\D/g, '') : '';
                        break;

                    case "DECIMAL":
                        if (typeof newValue === 'string') {
                            let m = newValue.match(/[0-9]*\.?[0-9]{0,2}/g)
                            if (m?.length > 0) {
                                c.value = m[0];
                                break;
                            }
                        }

                        c.value = '';
                        break;

                    case "BOOLEAN":
                        c.value = typeof newValue === 'boolean' ? newValue : true;
                        break;

                    case "DATETIME":
                        c.value = isValidDate(newValue) ? newValue : new Date();
                        break;

                    default:
                        c.value = newValue;
                        break;
                }
            }
        });

        setChangeState(changeList);

        return newValue;
    }

    function removeChange(change) {
        const changeList = [...changes];

        const index = changeList.indexOf(change);
        if (index > -1)
            changeList.splice(index, 1);

        setChangeState(changeList);
    }

    function isValidDate(date) {
        return date && Object.prototype.toString.call(date) === "[object Date]" && !isNaN(date);
    }

    function renderInput(entry) {
        switch (entry.type) {
            case "DATETIME":
                return (
                    <DatePicker
                        className="form-control"
                        name="value"
                        selected={entry.value}
                        //onSelect={handleDateSelect} //when day is clicked
                        onChange={(date) => setNewValue(date, entry)} //only when value has changed
                    />
                )

            case "BOOLEAN":
                return (
                    <input
                        type="checkbox"
                        name="value"
                        checked={entry.value}
                        onChange={(e) => setNewValue(!entry.value, entry)}
                        className="form-check-input"
                    />
                )

            case "ACCOUNT":
                return (
                    <Select onChange={(e) => setNewValue(e.value, entry)} name="value" options={accounts} isSearchable="true" isLoading={loading} />
                )

            case "ENVELOPE":
                return (
                    <Select onChange={(e) => setNewValue(e.value, entry)} name="value" options={envelopes} isSearchable="true" isLoading={loading} />
                )

            case "CURRENCYTYPE":
                return (
                    <Select onChange={(e) => setNewValue(e.value, entry)} name="value" options={currencyTypes} isSearchable="true" isLoading={loading} />
                )

            default:
                return (
                    <input className="form-control" type="text" name="value" value={entry.value} onChange={(e) => setNewValue(e.target.value, entry)}></input>    
                )
        }
    }

    function findValueInChangeFields(name) {
        let obj = fields.find(e => e.value.name === name)
        if (obj !== undefined && obj !== null) {
            return obj;
        }

        return {
            name: fields[0]?.value?.name,
            type: fields[0]?.value?.type
        }
    }

    function renderLogs() {
        console.log("Changes: ")
        console.log(changes);
    }

    return (
        <div>
            <p>{renderLogs()}</p>
            <table className="table table-sm table-striped">
                <thead>
                    <tr>
                        <th>Field</th>
                        <th>Value</th>
                        <th><RowButton type="button" className="btn btn-sm btn-outline-primary" text="Create Change" onClick={() => createChange()} /></th>
                    </tr>
                </thead>
                <tbody>
                    {
                        changes.map(entry => (                            
                            <tr key={entry.id}>
                                <td>
                                    {
                                        findValueInChangeFields(entry.field) !== undefined &&
                                        <Select onChange={(e) => setNewField(e.value, entry)} name="field" options={fields} isSearchable="true" isLoading={loading} value={findValueInChangeFields(entry.field)}/>
                                    }
                                </td>

                                <td>
                                    {renderInput(entry)}
                                </td>
                                <td><RowButton type="button" className="btn btn-sm btn-outline-danger" text="Delete" onClick={() => removeChange(entry)} /></td>
                            </tr>//*/
                        ))
                    }
                </tbody>
            </table>
        </div>
    );
}