import React from 'react';
import { EndPoints, getData } from "../services/AccessAPI";
//import { useNavigate } from "react-router-dom";
import { useState } from 'react';
import { useEffect } from "react";
import DatePicker from 'react-datepicker';
import Select from 'react-select';
import RowButton from "../ui-elements/table/RowButton";
import { GetUniqueRenderKey, StringTo2DigitDecimal, StringToInt } from '../helpers/DataValidation';
//import '../main.css';

export default function AutoclassExpressionBuilder({ expressions, setExpressionState, initialExpressions }) {
    useEffect(() => {
        //getExpressionData();
        getTypeOperatorInfoData();
        getFieldInfoData();

        getAllAccountsDataAsSelectInput();
        getAllEnvelopesDataAsSelectInput();
        getAllCurrencyTypeDataAsSelectInput();
    }, [])

    //let navigate = useNavigate();

    const [accounts, setAccounts] = useState([]);
    const [envelopes, setEnvelopes] = useState([]);
    const [currencyTypes, setCurrencyTypes] = useState([]);
    //const [expressions, setExpressionValues] = useState(expressions);
    const [fields, setFields] = useState([]);
    //const [fieldInfo, setFieldInfo] = useState([]);
    const [typeOperatorInfo, setTypeOperatorInfo] = useState([]);
    const [loading, setLoading] = useState(true);

    function getOperatorList(type) {
        for (let op of typeOperatorInfo) {
            if (op.type === type)
                return op.operators;
        }
    }


    useEffect(() => {
        if (initialExpressions === undefined)
            return;

        const expressionList = [...initialExpressions];

        expressionList.forEach(e => {

            let type = "";
            let operators = [];

            for (let f of fields) {
                if (f.label === e.transactionField) {
                    type = f.value.type;
                    break;
                }
            }

            for (let op of typeOperatorInfo) {
                if (op.type === type)
                {
                    operators = op.operators;
                    break;
                }
            }

            e.validOperators = operators.map(op => {
                return {
                    value: op.operator,
                    label: op.displayName
                }
            });
        });

        setExpressionState(expressionList);

    }, [initialExpressions, fields, typeOperatorInfo])

    function getAllAccountsDataAsSelectInput() {
        getData(EndPoints.ACCOUNTS).then(
            (result) => {
                if (result) {
                    setAccounts(result.map(a => {
                        return {
                            value: a.id,
                            label: a.name
                        }
                    }));
                    setLoading(false);
                }
            }
        );
    }

    function getAllEnvelopesDataAsSelectInput() {
        getData(EndPoints.ENVELOPES).then(
            (result) => {
                if (result) {
                    setEnvelopes(result.map(e => {
                        return {
                            value: e.id,
                            label: e.name
                        }
                    }));
                    setLoading(false);
                }
            }
        );
    }

    function getAllCurrencyTypeDataAsSelectInput() {
        getData(EndPoints.CURRENCYTYPES).then(
            (result) => {
                if (result) {
                    setCurrencyTypes(result.map(c => {
                        return {
                            value: c.id,
                            label: c.code
                        }
                    }));
                    setLoading(false);
                }
            }
        );
    }

    function getTypeOperatorInfoData() {
        getData(EndPoints.AUTOCLASS_TYPEOPERATORINFO).then(
            (result) => {
                if (result) {
                    setTypeOperatorInfo(result);
                    setLoading(false);
                }
            }
        );
    }

    function getFieldInfoData() {
        getData(EndPoints.AUTOCLASS_FIELDINFO).then(
            (result) => {
                if (result) {
                    setFields(result.map(a => {
                        return {
                            value: a,
                            label: a.name
                        }
                    }));

                    setLoading(false);
                }
            }
        );
    }

    function createExpression() {
        const expressionList = [...expressions];

        let exp = {
            id: GetUniqueRenderKey(),
            transactionField: fields[0].value.name,
            type: fields[0].value.type,
        }

        exp.validOperators = getValidOperators(exp.type);
        exp.compareOperator = exp.validOperators[0].value;
        exp.value = "";

        console.log(exp);

        expressionList.push(exp);
        setExpressionState(expressionList);
    }

    function setNewField(newField, expression) {
        const expressionList = [...expressions];

        expressionList.forEach(e => {
            if (e.id === expression.id) {
                e.transactionField = newField.name;
                e.type = newField.type;
                e.validOperators = getValidOperators(newField.type);
                setNewValue(e.value, expression)
            }
        });

        setExpressionState(expressionList);
    }

    function getValidOperators(type) {
        return getOperatorList(type).map(op => {
            return {
                value: op.operator,
                label: op.displayName
            }
        });
    }

    function setNewOperator(newOperator, expression) {
        const expressionList = [...expressions];

        expressionList.forEach(e => {
            if (e.id === expression.id) {
                e.compareOperator = newOperator;
            }
        });

        setExpressionState(expressionList);
    }

    function setNewValue(newValue, expression) {
        const expressionList = [...expressions];

        expressionList.forEach(e => {
            if (e.id === expression.id) {
                switch (e.type) {
                    case "INT":
                        e.value = StringToInt(newValue, '');
                        break;

                    case "DECIMAL":
                        e.value = StringTo2DigitDecimal(newValue, '');
                        break;

                    case "BOOLEAN":
                        e.value = typeof newValue === 'boolean' ? newValue : true;
                        break;

                    case "DATETIME":
                        e.value = isValidDate(newValue) ? newValue : new Date();
                        break;

                    default:
                        e.value = newValue;
                        break;
                }
            }
        });

        setExpressionState(expressionList);

        return newValue;
    }

    function removeExpression(exp) {
        const expressionList = [...expressions];

        const index = expressionList.indexOf(exp);
        if (index > -1)
            expressionList.splice(index, 1);

        setExpressionState(expressionList);
    }

    // Runs on render

    function isValidDate(date) {
        return date && Object.prototype.toString.call(date) === "[object Date]" && !isNaN(date);
    }

    function renderInput(entry) {
        switch (entry.type) {
            case "DATETIME":
                return (
                    <DatePicker
                        className="form-control"
                        name="value"
                        selected={entry.value}
                        //onSelect={handleDateSelect} //when day is clicked
                        onChange={(date) => setNewValue(date, entry)} //only when value has changed
                    />
                )

            case "BOOLEAN":
                return (
                    <input
                        type="checkbox"
                        name="value"
                        checked={entry.value}
                        onChange={(e) => setNewValue(!entry.value, entry)}
                        className="form-check-input"
                    />
                )

            case "ACCOUNT":
                return (
                    <Select onChange={(e) => setNewValue(e.value, entry)} name="value" options={accounts} isSearchable="true" isLoading={loading} />
                )

            case "ENVELOPE":
                return (
                    <Select onChange={(e) => setNewValue(e.value, entry)} name="value" options={envelopes} isSearchable="true" isLoading={loading} />
                )

            case "CURRENCYTYPE":
                return (
                    <Select onChange={(e) => setNewValue(e.value, entry)} name="value" options={currencyTypes} isSearchable="true" isLoading={loading} />
                )

            default:
                return (
                    <input className="form-control" type="text" name="value" value={entry.value} onChange={(e) => setNewValue(e.target.value, entry)}></input>    
                )
        }
    }
    
    // Used for Select boxes

    function findValueInExpressionFields(name) {
        let obj = fields.find(e => e.value.name === name)
        if (obj !== undefined && obj !== null) {
            return obj;
        }

        return fields[0];
    }

    function findValueInExpressionOperators(operators, name) {
        let obj = operators?.find(e => e.value === name)
        if (obj !== undefined && obj !== null) {
            return obj;
        }

        return operators[0];
    }

    return (
        <div>
            <table className="table table-sm table-striped">
                <thead>
                    <tr>
                        <th>Field</th>
                        <th>Operator</th>
                        <th>Value</th>
                        <th><RowButton type="button" className="btn btn-sm btn-outline-primary" text="CreateExp" onClick={() => createExpression()} /></th>
                    </tr>
                </thead>
                <tbody>
                    {
                        expressions.map(entry => (                            
                            <tr key={entry.id}>
                                <td>
                                    {
                                        findValueInExpressionFields(entry.transactionField) !== undefined &&
                                        <Select onChange={(e) => setNewField(e.value, entry)} name="transactionField" options={fields} isSearchable="true" isLoading={loading} value={findValueInExpressionFields(entry.transactionField)}/>
                                    }
                                </td>
                                <td>
                                    {
                                        findValueInExpressionOperators(entry.validOperators, entry.compareOperator) !== undefined &&
                                        <Select onChange={(e) => setNewOperator(e.value, entry)} name="compareOperator" options={entry.validOperators} isSearchable="true" isLoading={loading} value={findValueInExpressionOperators(entry.validOperators, entry.compareOperator)}/>
                                    }
                                </td>
                                <td>
                                    {renderInput(entry)}
                                </td>
                                <td><RowButton type="button" className="btn btn-sm btn-outline-danger" text="Delete" onClick={() => removeExpression(entry)} /></td>
                            </tr>//*/
                        ))
                    }
                </tbody>
            </table>
        </div>
    );
}