import SessionManager from "../auth/SessionManager";
import { EndPoints, putData, getData } from "../services/AccessAPI";
import { useNavigate, useLocation } from "react-router-dom";
import { useState } from 'react';
import { useEffect } from "react";
import { Routes } from "../services/Routes";
import Select from 'react-select';
//import DatePicker from 'react-datepicker';

import "react-datepicker/dist/react-datepicker.css";
import AutoclassExpressionBuilder from "./AutoclassExpressionBuilder";
import AutoclassChangeBuilder from "./AutoclassChangeBuilder";
import { findValueInArray } from "../helpers/SelectComponentUtils";
import { GetUniqueRenderKey } from "../helpers/DataValidation";

export default function UpdateAutoclassrule() {
    let navigate = useNavigate();
    const search = useLocation().search;
    const id = new URLSearchParams(search).get('id')

    useEffect(() => {
        getAutoclassData(id)
        getAllAccountsData();
    }, [id])

    const [accounts, setAccounts] = useState([]);
    const [name, setName] = useState("");
    const [chosenAccount, setChosenAccount] = useState(null);
    const [expressions, setExpressions] = useState([]);
    const [initialExpressions, setInitialExpressions] = useState([]);
    const [changes, setChanges] = useState([]);
    const [enabled, setEnabled] = useState(true);
    const [triggerOnUpdate, setTriggerOnUpdate] = useState(false);//TriggerOnCreate

    const [loading, setLoading] = useState(true);

    function onSubmit(e) {
        e.preventDefault();

        let autoclassruleObj = {
            name: name,
            //date: date,//"2023-12-23T19:29:12.909Z"
            accountId: chosenAccount,
            expressions: prepareExpressions(),
            changes: prepareChanges(),
            enabled: enabled,
            triggerOnUpdate: triggerOnUpdate,
        }

        console.log("Prepared payload")
        console.log(autoclassruleObj)

        putData(EndPoints.AUTOCLASS + "/" + id, autoclassruleObj).then((result) => {
            setLoading(false);
            let responseJson = result;
            if (responseJson) {
                navigate(Routes.ACCOUNTS_DASHBOARD + "?id=" + chosenAccount);
            }
        });
    }

    function prepareExpressions() {
        let expArray = [];

        expressions.forEach(e => {
            let newValue = e.value.toString();

            if (e.type === "DATETIME") {
                newValue = e.value.toISOString();
            }
            
            expArray.push({
                transactionField: e.transactionField,
                compareOperator: e.compareOperator,
                value: newValue
            })

            console.log("Expression Array: ");
            console.log(expArray);
        });

        return expArray;
    }

    function prepareChanges() {
        let chgArray = [];

        changes.forEach(c => {
            let newValue = c.value.toString();

            if (c.type === "DATETIME") {
                newValue = c.value.toISOString();
            }

            chgArray.push({
                field: c.field,
                value: newValue
            })
        });

        return chgArray;
    }

    function getAllAccountsData() {
        getData(EndPoints.ACCOUNTS).then(
            (result) => {
                if (result) {
                    setAccounts(result.map(a => {
                        return {
                            value: a.id,
                            label: a.name
                        }
                    }));
                    setLoading(false);
                }
            }
        );
    }

    function getAutoclassData(id) {
        getData(EndPoints.AUTOCLASS + "/" + id).then(
            (result) => {
                if (result) {
                    setName(result.name);
                    setChosenAccount(result.accountId);
                    setInitialExpressions(giveArrayUniqueIds(result.expressions));
                    setChanges(giveArrayUniqueIds(result.changes));
                    setEnabled(result.enabled);
                    setLoading(true);
                }
            }
        );
    }

    function giveArrayUniqueIds(array) {
        array.forEach(i => {
            i.id = GetUniqueRenderKey();
        })

        return array;
    }

    function onClickBack(e){
        e.preventDefault();

        if(SessionManager.getToken()){
            navigate(Routes.ACCOUNTS);
        }else{
            navigate(Routes.LOGIN);
        }
    }
        /*
            {
                "date": "2023-12-23T21:44:31.930Z",
                "isPending": true
            }
        */
    return (
        <div className="row">
            <div className="col-md-10">
                <h3>Update autoclass rule</h3>
                <form onSubmit={onSubmit}>
                    <div className="form-check">
                        <label>
                            <input
                                type="checkbox"
                                name="enabled"
                                checked={enabled}
                                onChange={(e) => setEnabled(!enabled)}
                                className="form-check-input"
                            />
                            Enabled
                        </label>
                    </div>

                    <div className="form-check">
                        <label>
                            <input
                                type="checkbox"
                                name="triggerOnUpdate"
                                checked={triggerOnUpdate}
                                onChange={(e) => setTriggerOnUpdate(!triggerOnUpdate)}
                                className="form-check-input"
                            />
                            Trigger After Update
                        </label>
                    </div>

                    <div className="form-group">
                        <label className="control-label">Name: </label>
                        <input className="form-control" type="text" name="name" value={name} onChange={(e) => setName(e.target.value)}></input>
                    </div>

                    {
                        findValueInArray(accounts, chosenAccount, -1) !== undefined && 
                        <div className="form-group">
                            <label className="control-label">Account: </label>
                            <Select onChange={(e) => setChosenAccount(e.value)} name="account" options={accounts} isSearchable="true" isLoading={loading} value={findValueInArray(accounts, chosenAccount)} />
                        </div>
                    }

                    <br />
                    <div>
                        <h3>Conditions</h3>
                        <p>All of these conditions must be met for the changes to take affect.</p>
                        {expressions.length}
                    </div>

                    <AutoclassExpressionBuilder
                        initialExpressions={initialExpressions}
                        expressions={expressions}
                        setExpressionState={setExpressions}
                    />

                    <br />
                    <div>
                        <h3>Changes</h3>
                        <p>All of these changes will occur when the rule triggers.</p>
                        {changes.length}
                    </div>

                    <AutoclassChangeBuilder
                        changes={changes}
                        setChangeState={setChanges}
                    />

                    {/*<div className="form-group">
                        <label className="control-label">Date: </label>
                        <DatePicker
                            className="form-control"
                            name="date"
                            selected={date}
                            onChange={(date) => updateDate(date)} //only when value has changed
                        />
                    </div>*/}

                    <div className="form-group">
                        <input type="submit" value="Update Autoclass Rule" className="btn btn-primary"></input> &nbsp; &nbsp; 
                        <input type="button" value="Back" onClick={onClickBack} className="btn btn-primary"></input>
                    </div>
                </form>
            </div>
        </div>
    );
}