import SessionManager from "../auth/SessionManager";
import { EndPoints, postData, getData } from "../services/AccessAPI";
import { useLocation, useNavigate } from 'react-router-dom';
import { useState, useEffect } from 'react';
import { Routes } from "../services/Routes";
import Select from 'react-select'
import FundingMethodBuilder from "./FundingMethodBuilder";
import TextInput from "../ui-elements/controls/input/TextInput";
import CheckBox from "../ui-elements/controls/input/CheckBox";

export default function CreateEnvelope() {
    const search = useLocation().search;
    const id = new URLSearchParams(search).get('id')
    const [showAccounts, setShowAccounts] = useState(true);

    useEffect(() => {
        getAllAccountsData();
        if (id !== undefined && id !== null) {
            setShowAccounts();
            setAccount(Number(id));
        }

    }, [id])

    let navigate = useNavigate();

    const [accounts, setAccounts] = useState([]);

    const [name, setName] = useState("");
    const [account, setAccount] = useState(0);
    const [initBalance, setInitBalance] = useState(0);
    const [enabled, setEnabled] = useState(true);
    const [isPersistant, setIsPersistant] = useState(true);
    const [priority, setPriority] = useState(0);
    const [extNum, setExtNum] = useState("");
    const [fundingMethods, setFundingMethods] = useState([]);

    const [loading, setLoading] = useState(true);

    function onSubmit(e) {
        e.preventDefault();

        let envelopeObj = {
            name: name,
            account: account,
            initialBalance: initBalance.toString(),
            enabled: enabled,
            isPersistant: isPersistant,
            priority: Number(priority),
            externalAccountNumber: extNum,
            fundingMethods: prepareFundingMethods(),
        }

        postData(EndPoints.ENVELOPES, envelopeObj).then((result) => {
            setLoading(false);
            let responseJson = result;
            if (responseJson) {
                navigate(Routes.ENVELOPES);
            }
        });
    }

    function prepareFundingMethods() {
        let methodArray = [];

        fundingMethods.forEach(e => {
            methodArray.push({
                order: e.order,
                mode: e.mode,
                value: e.value,
                periodsToLookback: e.lookBack
            })
        });

        return methodArray;
    }

    function getAllAccountsData() {
        getData(EndPoints.ACCOUNTS).then(
            (result) => {
                if (result) {
                    setAccounts(result.map(a => {
                        return {
                            value: a.id,
                            label: a.name
                        }
                    }));
                    setLoading(false);
                }
            }
        );
    }

    function onClickBack(e){
        e.preventDefault();

        if(SessionManager.getToken()){
            navigate(Routes.ENVELOPES);
        }else{
            navigate(Routes.LOGIN);
        }   
    }

    return (
        <div className="row">
            <div className="col-md-12">
                <h3>Create new Envelope</h3>
                <hr />
                <form onSubmit={onSubmit}>
                    <div className="row">
                        <div className="col-md-4">
                            <div className="form-group">
                                <label className="control-label">Envelope Name: </label>
                                <TextInput
                                    className="form-control"
                                    value={name}
                                    onChange={(e) => setName(e.target.value)}
                                    name="name"
                                />
                            </div>

                            {showAccounts && <div className="form-group">
                                <label className="control-label">Account: </label>
                                <Select className="aa-select" onChange={(e) => setAccount(e.value)} name="account" options={accounts} isSearchable="true" isLoading={loading} />
                            </div>}

                            <div className="form-group">
                                <label className="control-label">Initial Balance: </label>
                                <TextInput
                                    className="form-control"
                                    value={initBalance}
                                    onChange={(e) => setInitBalance(e.target.value)}
                                    name="initBalance"
                                />
                            </div>

                            <div className="form-group">
                                <label className="control-label">Priority: </label>
                                <TextInput
                                    className="form-control"
                                    value={priority}
                                    onChange={(e) => setPriority(e.target.value)}
                                    name="priority"
                                />
                            </div>
                            
                            <div className="form-check">
                                <label>
                                    <input
                                        type="checkbox"
                                        name="enabled"
                                        checked={enabled}
                                        onChange={(e) => setEnabled(!enabled)}
                                        className="form-check-input"
                                    />
                                    Enabled
                                </label>
                            </div>
                            
                            <div className="form-check">
                                <label>
                                    <input
                                        type="checkbox"
                                        name="isPersistant"
                                        checked={isPersistant}
                                        onChange={(e) => setIsPersistant(!isPersistant)}
                                        className="form-check-input"
                                    />
                                    Is Persistant
                                </label>
                            </div>

                            <div className="form-group">
                                <label className="control-label">External Account Number: </label>
                                <TextInput
                                    className="form-control"
                                    value={extNum}
                                    onChange={(e) => setExtNum(e.target.value)}
                                    name="extNum"
                                />
                            </div>

                            <br />

                            <div className="form-group">
                                <input type="button" value="Back" onClick={onClickBack} className="btn btn-primary"></input>
                                &nbsp;&nbsp;
                                <input type="submit" value="Create Envelope" className="btn btn-success"></input>
                            </div>
                        </div>

                        <div className="col-md-8">
                            <div>
                                <h4>Funding Methods</h4>
                                <p>Each month, these operations will trigger and transfer from the Account's virtualBalance.</p>
                                {fundingMethods.length}
                            </div>

                            <FundingMethodBuilder
                                fundingMethods={fundingMethods}
                                setFundingMethods={setFundingMethods}
                            />
                        </div>
                    </div>
                </form>
            </div>
        </div>
    );
}