import { EndPoints, getData } from "../services/AccessAPI";
import { useNavigate } from "react-router-dom";
import { useEffect } from "react";
import { useState } from 'react';
import { Routes } from "../services/Routes";
import RowButton from "../ui-elements/table/RowButton";
import DataFormatter, { RenderType } from "../ui-elements/table/DataFormatter";
import EditButton from "../ui-elements/controls/buttons/EditButton";
import DeleteButton from "../ui-elements/controls/buttons/DeleteButton";

export default function Envelopes() {
    useEffect(() => {;
        getAllAccountsData();
        getAllEnvelopesData();
        setLoading(false);
    }, [])

    let navigate = useNavigate();

    const [loading, setLoading] = useState(true);
    const [envelopes, setEnvelopes] = useState([]);
    const [accounts, setAccounts] = useState([]);

    function getAllEnvelopesData() {
        getData(EndPoints.ENVELOPES).then(
            (result) => {
                if (result) {
                    setEnvelopes(result);
                }
            }
        );
    }

    function getAllAccountsData() {
        getData(EndPoints.ACCOUNTS).then(
            (result) => {
                if (result) {
                    setAccounts(result);
                }
            }
        );
    }

    function findAccountName(id) {
        let name = accounts?.find(x => x.id === id)?.name;
        console.log("Id: " + id + ", Name: " + name);
        return name;
    }

    function renderAllEnvelopesTable() {
        return (
            <div className="container-fluid">
                <div className="row">
                    <div className="col-md-12">
                        <table className="table table-sm table-striped">
                            <thead>
                                <tr>
                                    <th>Name</th>
                                    <th>Account</th>
                                    <th>Balance</th>
                                    <th>Initial Balance</th>
                                    <th>Status</th>
                                    <th>Persistant</th>
                                    <th>Priority</th>
                                    <th>Actions</th>
                                </tr>
                            </thead>
                            <tbody>
                                {
                                    envelopes.map(env => (
                                        <tr key={env.id}>
                                            <td><DataFormatter data={env.name} /></td>
                                            <td><DataFormatter data={findAccountName(env.accountId)} /></td>
                                            <td><DataFormatter data={env.balance} renderType={RenderType.MoneyUnsigned}/></td>
                                            <td><DataFormatter data={env.initialBalance} renderType={RenderType.MoneyUnsigned}/></td>
                                            <td><DataFormatter data={env.enabled ? "Enabled" : "Disabled"} /></td>
                                            <td><DataFormatter data={env.isPersistant ? "True" : "False"} /></td>
                                            <td><DataFormatter data={env.priority} /></td>
                                            <td>
                                                <EditButton
                                                    className="btn btn-sm btn-primary"
                                                    onClick={() => navigate(Routes.ENVELOPES_EDIT + "?id=" + env.id)}
                                                />
                                                &nbsp;
                                                <DeleteButton
                                                    className="btn btn-sm btn-danger"
                                                    onClick={() => navigate(Routes.ENVELOPES_DELETE + "?id=" + env.id)}
                                                />
                                            </td>
                                        </tr>
                                    ))
                                }
                            </tbody>
                        </table>
                    </div>
                </div>
            </div>
        );
    }

    let content = loading ? (
        <p>
            <em>Loading...</em>
        </p>
    ) : (
        renderAllEnvelopesTable()
    )

    
    return (
        <div>
            <h3>List of Envelopes</h3>
            <button onClick={() => navigate(Routes.ENVELOPES_CREATE)} className="btn btn-primary">Create new envelope</button>
            {content}
        </div>
    );
}