import React from 'react';
import { EndPoints, getData } from "../services/AccessAPI";
import { useState } from 'react';
import { useEffect } from "react";
import Select from 'react-select';
import RowButton from "../ui-elements/table/RowButton";
import { StringTo2DigitDecimal, StringToInt } from '../helpers/DataValidation';
import { findValueInArray } from "../helpers/SelectComponentUtils";
//import '../main.css';

export default function FundingMethodBuilder({ fundingMethods, setFundingMethods }) {
    useEffect(() => {
        getFundingMethodModes();
    }, [])

    const [fundingMethodModes, setFundingMethodModes] = useState([]);
    
    const [loading, setLoading] = useState(true);

    function getFundingMethodModes() {
        getData(EndPoints.ENVELOPES_FUNDINGMETHODMODES).then(
            (result) => {
                if (result) {
                    setFundingMethodModes(result.map(a => {
                        return {
                            value: a,
                            label: prettyUpModes(a)
                        }
                    }));
                    setLoading(false);
                }
            }
        );
    }

    function prettyUpModes(modeName)
    {
        return modeName.replace("_", " ");
    }

    function createFundingMethod() {
        const methods = [...fundingMethods];

        let method = {
            id: Date.now(),
            order: 10,
            mode: null,
            value: "",
            lookBack: 3,
        }

        methods.push(method);
        setFundingMethods(methods);
    }

    function setNewMode(newMode, method) {
        const methods = [...fundingMethods];

        methods.forEach(e => {
            if (e.id === method.id) {
                e.mode = newMode;
            }
        });

        setFundingMethods(methods);
    }

    function setNewOrder(newOrder, method) {
        const methods = [...fundingMethods];

        methods.forEach(e => {
            if (e.id === method.id) {
                e.order = StringToInt(newOrder, "");
            }
        });

        setFundingMethods(methods);
    }

    function setNewLookBack(newLookBack, method) {
        const methods = [...fundingMethods];

        methods.forEach(e => {
            if (e.id === method.id) {
                e.lookBack = StringToInt(newLookBack, "");
            }
        });

        setFundingMethods(methods);
    }

    function setNewValue(newValue, method) {
        const methods = [...fundingMethods];

        methods.forEach(e => {
            if (e.id === method.id) {
                e.value = StringTo2DigitDecimal(newValue, "");
            }
        });

        setFundingMethods(methods);
    }

    function removeFundingMethod(method) {
        const methods = [...fundingMethods];

        const index = methods.indexOf(method);
        if (index > -1)
            methods.splice(index, 1);

        setFundingMethods(methods);
    }

    return (
        <div>
            <table className="table table-sm table-striped">
                <thead>
                    <tr>
                        <th style={{ width: "60px" }}>Order</th>
                        <th>Mode</th>
                        <th style={{ width: "120px" }}>Value</th>
                        <th>LookBack in Months</th>
                        <th><RowButton type="button" className="btn btn-sm btn-outline-primary" text="Create Method" onClick={() => createFundingMethod()} /></th>
                    </tr>
                </thead>
                <tbody>
                    {
                        fundingMethods.map(entry => (                            
                            <tr key={entry.id}>
                                <td>
                                    <input className="form-control" type="text" name="order" value={entry.order} onChange={(e) => setNewOrder(e.target.value, entry)}></input>
                                </td>

                                <td>
                                {
                                        findValueInArray(fundingMethodModes, entry.mode, -1) !== undefined &&
                                        <Select onChange={(e) => setNewMode(e.value, entry)} name="mode" options={fundingMethodModes} isSearchable="true" isLoading={loading} value={findValueInArray(fundingMethodModes, entry.mode)}/>
                                }
                                </td>

                                <td>
                                    <input className="form-control" type="text" name="value" value={entry.value} onChange={(e) => setNewValue(e.target.value, entry)}></input>
                                </td>

                                <td>
                                    <input className="form-control" type="text" name="lookBack" value={entry.lookBack} onChange={(e) => setNewLookBack(e.target.value, entry)}></input>
                                </td>

                                <td>
                                    <RowButton type="button" className="btn btn-sm btn-outline-danger" text="Delete" onClick={() => removeFundingMethod(entry)} />
                                </td>
                            </tr>//*/
                        ))
                    }
                </tbody>
            </table>
        </div>
    );
}