import SessionManager from "../auth/SessionManager";
import { DEV_URL } from "./Settings";

export const EndPoints = {
    USERS: "Users",
    USER_AUTHENTICATE: "Users/authenticate",
    USER_CANREGISTER: "Users/CanRegister",
    USER_REGISTER: "Users/register",
    ACCOUNTS: "Accounts",
    ACCOUNTS_REFRESHBALANCE: "Accounts/RefreshBalance",
    ACCOUNTS_HISTORICAL: "Accounts/Historical",
    CURRENCYTYPES: "CurrencyTypes",
    TRANSACTIONS: "Transactions",
    ENVELOPES: "Envelopes",
    ENVELOPES_HISTORICAL: "Envelopes/Historical",
    ENVELOPES_FUNDINGMETHODMODES: "Envelopes/FundingMethodModes",
    AUTOCLASS: "Autoclass",
    AUTOCLASS_TYPEOPERATORINFO: "Autoclass/TypeOperatorInfo",
    AUTOCLASS_FIELDINFO: "Autoclass/FieldInfo",
    AUTOCLASS_TRIGGERAUTOCLASSRULE: "Autoclass/TriggerAutoclassRule",
    AUTOCLASS_TRIGGERTRANSACTIONRULES: "Autoclass/TriggerTransactionRules",
    APIKEY: "ApiKey",
    APIKEY_CREATENEW: "ApiKey/CreateNew",
    APPVERSION: "AppInfo/ApplicationVersion",
}

function getOrigin() {
    if (window.location.href.includes("http://localhost") || window.location.href.includes("https://localhost"))
        return DEV_URL;
    else
        return window.location.origin + "/"
}

export function getData(endPoint) {

    let token=SessionManager.getToken();

    let payload = {
        method: 'GET',
        headers: {   
            "Access-Control-Allow-Origin" : "*", 
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + token
         },
    }
    return fetch(getOrigin() + endPoint, payload)
    .then(function(response) {
        if (!response.ok) {
            throw Error(response.statusText);
        }
        return response.json();
    }).then(function(result) {
        return result;
    }).catch(function(error) {
        console.log(error);
    });
}

export function postDataForLogin(type, userData) {
    //let BaseURL = "https://localhost:7142/";
    let payload = {
        method: 'POST',
        headers: {   
            "Access-Control-Allow-Origin" : "*",
            'Content-Type': 'application/json' 
        },
        body: JSON.stringify(userData)

    }
    return fetch(getOrigin() + type, payload)
    .then(function(response) {
        return response.json();
    }).then(function(result) {
        return result;
    }).catch(function(error) {
        console.log(error);
    });
}

export function postData(endPoint, inputObj) {
    let token=SessionManager.getToken();
    let payload = {
        method: 'POST',
        headers: {   
            "Access-Control-Allow-Origin" : "*", 
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + token
        },
        body: JSON.stringify(inputObj)

    }
    return fetch(getOrigin() + endPoint, payload)
    .then(function(response) {
        return response.json();
    }).then(function(result) {
        return result;
    }).catch(function(error) {
        console.log(error);
    });
}

export function deleteData(endPoint) {
    let token=SessionManager.getToken();
    let payload = {
        method: 'DELETE',
        headers: {   
            "Access-Control-Allow-Origin" : "*", 
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + token
         },
    }
    return fetch(getOrigin() + endPoint, payload)
    .then(function(response) {
        if (!response.ok) {
            throw Error(response.statusText);
        }
        return response.json();
    }).then(function(result) {
        return result;
    }).catch(function(error) {
        console.log(error);
    });
}

export function putData(endPoint, obj) {
    let token=SessionManager.getToken();
    let payload = {
        method: 'PUT',
        headers: {   
            "Access-Control-Allow-Origin" : "*", 
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + token
        },
        body: JSON.stringify(obj)

    }
    return fetch(getOrigin() + endPoint, payload)
    .then(function(response) {
        return response.json();
    }).then(function(result) {
        return result;
    }).catch(function(error) {
        console.log(error);
    });
}
