import { useNavigate } from "react-router-dom";
import { EndPoints, deleteData, getData } from "../services/AccessAPI";
import { useEffect } from "react";
import { useState } from 'react';
import { useLocation } from 'react-router-dom';
import { Routes } from "../services/Routes";
import DataFormatter, { RenderType } from "../ui-elements/table/DataFormatter";

export default function DeleteTransaction() {
    const search = useLocation().search;
    const id = new URLSearchParams(search).get('id')

    useEffect(() => {
        getData(EndPoints.TRANSACTIONS + "/" + id).then(
            (result) => {
                if (result) {
                    setState({
                        amount: result.amount,
                        description: result.description,
                        date: result.date,
                        externalId: result.externalId,
                        debitAccountId: result.debitAccountId,
                        creditAccountId: result.creditAccountId,
                        isPending: result.isPending,
                        loading: false
                    });
                }
            }
        );
    }, [id])

    let navigate = useNavigate();

    const [state, setState] = useState({
        amount: 0,
        description: '',
        date: '',
        externalId: '',
        debitAccountId: 0,
        creditAccountId: 0,
        isPending: false,
        loading: true
    });

    function onCancel() {
        navigate(Routes.TRANSACTIONS);
    }

    function onConfirmation(e) {
        e.preventDefault();

        deleteData(EndPoints.TRANSACTIONS + "/" + id).then((result) => {
            let responseJson = result;
            if (responseJson) {
                navigate(Routes.TRANSACTIONS);
            }
        }
        );

    }
    
    return (
        <div>
            <h2>::Delete Transaction::</h2>
            <h3>Are you sure you want to delete this?</h3>
            <div>
                <h4>Transaction Information</h4>
                <dl className="row">
                    <dt className="col-sm-2">
                        Transaction Description:
                    </dt>
                    <dd className="col-sm-10">
                        {state.description}
                    </dd>
                </dl>

                <dl className="row">
                    <dt className="col-sm-2">
                        Amount:
                    </dt>
                    <dd className="col-sm-10">
                        <DataFormatter data={state.amount} renderType={RenderType.MoneyUnsigned}/>
                    </dd>
                </dl>

                <dl className="row">
                    <dt className="col-sm-2">
                        Date:
                    </dt>
                    <dd className="col-sm-10">
                        {state.date}
                    </dd>
                </dl>

                <dl className="row">
                    <dt className="col-sm-2">
                        Debit Account:
                    </dt>
                    <dd className="col-sm-10">
                        {state.debitAccountId}
                    </dd>
                </dl>

                <dl className="row">
                    <dt className="col-sm-2">
                        Credit Account:
                    </dt>
                    <dd className="col-sm-10">
                        {state.creditAccountId}
                    </dd>
                </dl>

                <dl className="row">
                    <dt className="col-sm-2">
                        Is Pending:
                    </dt>
                    <dd className="col-sm-10">
                        <DataFormatter data={state.isPending ? "True" : "False"} />
                    </dd>
                </dl>

                <dl className="row">
                    <dt className="col-sm-2">
                        External Transaction Number:
                    </dt>
                    <dd className="col-sm-10">
                        {state.externalId}
                    </dd>
                </dl>

                <form onSubmit={onConfirmation}>
                    <input type="hidden" asp-for="Id" />
                    <button onClick={onCancel} className="btn btn-primary">Back</button>
                    &nbsp;&nbsp;
                    <button type="submit" className="btn btn-danger">Delete</button>
                </form>
            </div>
        </div>
    )
}