import SessionManager from "../auth/SessionManager";
import { EndPoints, putData, getData } from "../services/AccessAPI";
import { useNavigate, useLocation } from "react-router-dom";
import { useState } from 'react';
import { useEffect } from "react";
import { Routes } from "../services/Routes";
import Select from 'react-select';
import DatePicker from 'react-datepicker';

import "react-datepicker/dist/react-datepicker.css";
import TextInput from "../ui-elements/controls/input/TextInput";

export default function UpdateTransaction() {
    let navigate = useNavigate();
    const search = useLocation().search;
    const id = new URLSearchParams(search).get('id')

    useEffect(() => {
        getTransactionData(id);
        getAllAccountsData();
        getAllEnvelopeData();
    }, [id])

    const [accounts, setAccounts] = useState([]);
    const [envelopes, setEnvelopes] = useState([]);

    // Transaction Update Request Fields
    const [date, setDate] = useState("");
    const [extNum, setExtNum] = useState("");
    const [description, setDescription] = useState("");
    const [debitAccount, setDebitAccount] = useState(null);
    const [creditAccount, setCreditAccount] = useState(null);
    const [debitEnvelope, setDebitEnvelope] = useState(null);
    const [creditEnvelope, setCreditEnvelope] = useState(null);
    const [amount, setAmount] = useState(0);
    //const [currencyType, setCurrencyType] = useState(0);
    const [notes, setNotes] = useState("");
    const [isPending, setIsPending] = useState(false);
    const [triggerAutoclassRules, setTriggerAutoclassRules] = useState(false);



    const [loading, setLoading] = useState(true);

    function onSubmit(e) {
        e.preventDefault();

        let transactionObj = {
            //owner: SessionManager.getUserId(),
            date: date,//"2023-12-23T19:29:12.909Z"
            externalId: extNum,
            description: description,
            debitAccount: debitAccount,
            creditAccount: creditAccount,
            debitEnvelope: debitEnvelope,
            creditEnvelope: creditEnvelope,
            amount: amount,
            currencyType: 1,
            notes: notes,
            isPending: isPending
        }

        putData(EndPoints.TRANSACTIONS + "/" + id, transactionObj).then((result) => {
            setLoading(false);
            let responseJson = result;

            if (responseJson) {
                console.log(responseJson);
                navigate(Routes.TRANSACTIONS);
            }
        });
    }

    function getTransactionData(id) {
        getData(EndPoints.TRANSACTIONS + "/" + id).then(
            (result) => {
                if (result) {
                    setDate(new Date(result.date));
                    setExtNum(result.externalId);
                    setDescription(result.description);
                    setDebitAccount(result.debitAccountId);
                    setCreditAccount(result.creditAccountId);
                    setDebitEnvelope(result.debitEnvelopeId);
                    setCreditEnvelope(result.creditEnvelopeId);
                    setAmount(result.amount);
                    //setCurrencyType(0);
                    setNotes(result.notes);
                    setIsPending(result.isPending);
                    setTriggerAutoclassRules(result.setTriggerAutoclassRules);

                    setLoading(false);
                    console.log("State set");
                }
            }
        );
    }

    function getAllAccountsData() {
        getData(EndPoints.ACCOUNTS).then(
            (result) => {
                if (result) {
                    let accountList = [{
                        value: 0,
                        label: "(void)"
                    }]
                    
                    accountList.push.apply(
                        accountList,
                        result.map(a => {
                            return {
                                value: a.id,
                                label: a.name
                            }
                        }
                    ));
                    
                    setAccounts(accountList);
                    setLoading(false);
                }
            }
        );
    }

    function getAllEnvelopeData() {
        getData(EndPoints.ENVELOPES).then(
            (result) => {
                if (result) {
                    let envelopeList = [{
                        value: 0,
                        label: "(void)"
                    }]
                    
                    envelopeList.push.apply(
                        envelopeList,
                        result.map(a => {
                            return {
                                value: a.id,
                                label: a.name
                            }
                        }
                    ));
                    
                    setEnvelopes(envelopeList);
                    setLoading(false);
                }
            }
        );
    }

    function onClickBack(e){
        e.preventDefault();

        if(SessionManager.getToken()){
            navigate(Routes.TRANSACTIONS);
        }else{
            navigate(Routes.LOGIN);
        }
    }

    function updateDate(date) {
        setDate(date);
    }

    function parseAccountValue(id)
    {
        return accounts.find(e => e.value === id);
    }

    function parseEnvelopeValue(id)
    {
        return envelopes.find(e => e.value === id);
    }
    
    return (
        <div className="row">
            <div className="col-md-4">
                <h3>Update transaction</h3>
                <form onSubmit={onSubmit}>
                    <div className="form-group">
                        <label className="control-label">Description: </label>
                        <TextInput
                            className="form-control"
                            value={description}
                            onChange={(e) => setDescription(e.target.value)}
                            name="description"
                        />
                    </div>

                    <div className="form-group">
                        <label className="control-label">Amount: </label>
                        <TextInput
                            className="form-control"
                            value={amount}
                            onChange={(e) => setAmount(e.target.value)}
                            name="amount"
                        />
                    </div>

                    { parseAccountValue(debitAccount) !== undefined && 
                        <div className="form-group">
                            <label className="control-label">Debit Account: </label>
                            <Select onChange={(e) => setDebitAccount(e.value)} name="debitAccount" options={accounts} isSearchable="true" isLoading={loading} value={parseAccountValue(debitAccount)} />
                        </div>
                    }

                    { parseAccountValue(creditAccount) !== undefined && 
                        <div className="form-group">
                            <label className="control-label">Credit Account: </label>
                            <Select onChange={(e) => setCreditAccount(e.value)} name="creditAccount" options={accounts} isSearchable="true" isLoading={loading} value={parseAccountValue(creditAccount)} />
                        </div>
                    }

                    { parseEnvelopeValue(debitEnvelope) !== undefined && 
                        <div className="form-group">
                            <label className="control-label">Debit Envelope: </label>
                            <Select onChange={(e) => setDebitEnvelope(e.value)} name="debitEnvelope" options={envelopes} isSearchable="true" isLoading={loading} value={parseEnvelopeValue(debitEnvelope)} />
                        </div>
                    }

                    { parseEnvelopeValue(creditEnvelope) !== undefined && 
                        <div className="form-group">
                            <label className="control-label">Credit Envelope: </label>
                            <Select onChange={(e) => setCreditEnvelope(e.value)} name="creditEnvelope" options={envelopes} isSearchable="true" isLoading={loading} value={parseEnvelopeValue(creditEnvelope)} />
                        </div>
                    }

                    <div className="form-group">
                        <label className="control-label">Date: </label>
                        {/*<DatePicker selected={date} onChange={(date) => setDate(date)} />*/}

                        <DatePicker
                            className="form-control"
                            name="date"
                            selected={date}
                            //onSelect={handleDateSelect} //when day is clicked
                            onChange={(date) => updateDate(date)} //only when value has changed
                        />

                        {/*<input className="form-control" type="text" name="date" value={date} onChange={(e) => setDate(e.target.value)}></input>*/}
                    </div>                    

                    <div className="form-group">
                        <label className="control-label">External Transaction Number: </label>
                        <TextInput
                            className="form-control"
                            value={extNum}
                            onChange={(e) => setExtNum(e.target.value)}
                            name="extNum"
                        />
                    </div>

                    <div className="form-group">
                        <label className="control-label">Notes: </label>
                        <TextInput
                            className="form-control"
                            value={notes}
                            onChange={(e) => setNotes(e.target.value)}
                            name="notes"
                        />
                    </div>
                    
                    <div className="form-check">
                        <label>
                            <input
                                type="checkbox"
                                name="isPending"
                                checked={isPending}
                                onChange={(e) => setIsPending(!isPending)}
                                className="form-check-input"
                            />
                            Is Pending
                        </label>
                    </div>
                    
                    <div className="form-check">
                        <label>
                            <input
                                type="checkbox"
                                name="triggerAutoclassRules"
                                checked={triggerAutoclassRules}
                                onChange={(e) => setTriggerAutoclassRules(!triggerAutoclassRules)}
                                className="form-check-input"
                            />
                            Trigger Autoclass Rules
                        </label>
                    </div>

                    <div className="form-group">
                        <input type="button" value="Cancel" onClick={onClickBack} className="btn btn-primary"></input>
                        &nbsp;&nbsp;
                        <input type="submit" value="Update Transaction" className="btn btn-warning"></input>
                    </div>
                </form>
            </div>
        </div>
    );
}