import React from 'react';
import { EndPoints, getData } from "../../services/AccessAPI";
import { useNavigate } from "react-router-dom";
import { useEffect } from "react";
import { useState } from 'react';
import { Badge } from 'reactstrap';
import { Routes } from "../../services/Routes";
import RowButton from "./RowButton";
import DataFormatter, { RenderType } from "./DataFormatter";
import EditButton from '../controls/buttons/EditButton';
import DeleteButton from '../controls/buttons/DeleteButton';

export default function TranactionTable({ transactions, setTransactions, referenceAccountId, signedAmount, className, disabled, type }) {
    useEffect(() => {
        getAllAccountsData();
        getAllEnvelopesData();
    }, [])

    let navigate = useNavigate();

    const [loading, setLoading] = useState(true);
    const [accounts, setAccounts] = useState([]);
    const [envelopes, setEnvelopes] = useState([]);

    const [showCredit, setShowCredit] = useState(true);
    const [showDebit, setShowDebit] = useState(true);
    const [showPending, setShowPending] = useState(true);

    function getAllAccountsData() {
        getData(EndPoints.ACCOUNTS).then(
            (result) => {
                if (result) {
                    setAccounts(result);
                    setLoading(false);
                }
            }
        );
    }

    function getAllEnvelopesData() {
        getData(EndPoints.ENVELOPES).then(
            (result) => {
                if (result) {
                    setEnvelopes(result);
                    setLoading(false);
                }
            }
        );
    }

    function onTransactionEdit(id){
        let query = "?id=" + id
        navigate(Routes.TRANSACTIONS_EDIT + query);
    }

    function onTransactionDelete(id){
        let query = "?id=" + id
        navigate(Routes.TRANSACTIONS_DELETE + query);
    }

    function findAccountName(id) {
        let name = accounts?.find(x => x.id === id)?.name;
        return name;
    }

    function findEnvelopeName(id) {
        let name = envelopes?.find(x => x.id === id)?.name;
        console.log("Id: " + id + ", Name: " + name);
        return name;
    }

    function truncate(data, n) {
        if (data !== null && data !== undefined)
            return (data.length > n) ? data.slice(0, n-1) : data;
        else
            return "";
    };

    function isAccountTable()
    {
        return referenceAccountId !== null && referenceAccountId !== undefined;
    }

    function adjustAmountForReferenceAccount(amount, accountId)
    {
        if (referenceAccountId === null || referenceAccountId === undefined)
            return amount;

        return accountId === Number(referenceAccountId) ? -amount : amount;
    }

    function getRowClass(transaction)
    {
        if (transaction.isPending)
            return "pending";

        return "";
    }

    return (
        <>
            <button onClick={() => setShowCredit(!showCredit)} className="btn btn-sm btn-outline-primary">
                {showCredit ? "Hide Credit" : "Show Credit"}
            </button>

            &nbsp;

            <button onClick={() => setShowDebit(!showDebit)} className="btn btn-sm btn-outline-primary">
                {showDebit ? "Hide Debit" : "Show Debit"}
            </button>

            &nbsp;

            <button onClick={() => setShowPending(!showPending)} className="btn btn-sm btn-outline-primary">
                {showPending ? "Hide Pending" : "Show Pending"}
            </button>
        
            <table className="transaction-table table table-sm table-striped">
                <thead>
                    <tr>
                        <th style={{ width: "50px" }}>Status</th>
                        <th style={{ width: "70px" }}>Amount</th>
                        <th>Description</th>
                        <th style={{ width: "150px" }}>External ID</th>
                        {
                            isAccountTable()
                            ?
                            (<th style={{ width: "100px" }}>Other Account</th>)
                            :
                            (<>
                                <th style={{ width: "100px" }}>Debit Account</th>
                                <th style={{ width: "5px" }}></th>
                                <th style={{ width: "100px" }}>Credit Account</th>
                            </>)
                        }
                        <th style={{ width: "100px" }}>Debit Envelope</th>
                        <th style={{ width: "5px" }}></th>
                        <th style={{ width: "100px" }}>Credit Envelope</th>
                        <th style={{ width: "70px" }}>Date</th>
                        <th style={{ width: "70px" }}>Actions</th>
                    </tr>
                </thead>
                <tbody>
                    {
                        transactions.map(transaction => {
                            let amount = adjustAmountForReferenceAccount(transaction.amount, transaction.debitAccountId);

                            let renderThisTransaction = true;

                            if (amount < 0 && showDebit === false)
                                renderThisTransaction = false;
                            else if (amount >= 0 && showCredit === false)
                                renderThisTransaction = false;

                            if (showPending === false && transaction.isPending)
                                renderThisTransaction = false;

                            return (renderThisTransaction &&
                            <tr className={getRowClass(transaction)} key={transaction.id}>
                                <td>{transaction.isPending ?
                                <Badge
                                    color='warning'
                                    className="text-dark">
                                    Pending
                                </Badge> : "" }</td>
                                <td className='row-amount'><DataFormatter data={amount} renderType={signedAmount ? RenderType.MoneySigned : RenderType.MoneyUnsigned}/></td>
                                <td><DataFormatter data={transaction.description} /></td>
                                <td><DataFormatter data={transaction.externalId} /></td>
                                {
                                    isAccountTable()
                                    ?
                                    (<td><DataFormatter data={truncate(findAccountName(transaction.creditAccountId == referenceAccountId ? transaction.debitAccountId : transaction.creditAccountId), 16)} /></td>)
                                    :
                                    (<>

                                        <td><DataFormatter data={truncate(findAccountName(transaction.debitAccountId), 16)} /></td>
                                        <td>{"=>"}</td>
                                        <td><DataFormatter data={truncate(findAccountName(transaction.creditAccountId), 16)} /></td>
                                    </>)
                                }
                                <td><DataFormatter data={truncate(findEnvelopeName(transaction.debitEnvelopeId), 16)} /></td>
                                <td>{"=>"}</td>
                                <td><DataFormatter data={truncate(findEnvelopeName(transaction.creditEnvelopeId), 16)} /></td>
                                <td><DataFormatter data={transaction.date} renderType={RenderType.Date} /></td>
                                <td>
                                    <EditButton
                                        className="row-btn btn btn-sm btn-primary"
                                        onClick={() => onTransactionEdit(transaction.id)}
                                    />
                                    &nbsp;
                                    <DeleteButton
                                        className="row-btn btn btn-sm btn-danger"
                                        onClick={() => onTransactionDelete(transaction.id)}
                                    />
                                </td>
                            </tr>)
                        })
                    }
                </tbody>
            </table>
        </>
    );
}