import SessionManager from "../auth/SessionManager";
import { EndPoints, postData } from "../services/AccessAPI";
import { useNavigate } from "react-router-dom";
import { useState } from 'react';
import { Routes } from "../services/Routes";
import TextInput from "../ui-elements/controls/input/TextInput";

export default function CreateUser() {
    let navigate = useNavigate();

    const [firstName, setFirstName] = useState("");
    const [lastName, setLastName] = useState("");
    const [email, setEmail] = useState("");
    const [username, setUsername] = useState("");
    const [password, setPassword] = useState("");
    const [confirmationPassword, setConfirmationPassword] = useState("");
    const [role, setRole] = useState("");
    const [loading, setLoading] = useState(true);

    function onSubmit(e) {
        e.preventDefault();

        if (password !== confirmationPassword) {
            alert("Password and confirm password are not same");
            return;
        }

        let userObj = {
            firstName: firstName,
            lastName: lastName,
            email: email,
            username: username,
            password: password,
            role: role,
        }

        postData(EndPoints.USER_REGISTER, userObj).then((result) => {
            setLoading(false);
            let responseJson = result;
            if (responseJson) {
                navigate(Routes.USERS);
            }
        });
    }

    function onClickBack(e){
        e.preventDefault();

        if(SessionManager.getToken()){
            navigate(Routes.USERS);
        }else{
            navigate(Routes.LOGIN);
        }   
    }

    /*function onChange(e) {
        //setState({ [e.target.name]: e.target.value });
    }*/

    return (
        <div className="row">
            <div className="col-md-4">
                <h3>Create new user</h3>
                <form onSubmit={onSubmit}>
                    <div className="form-group">
                        <label className="control-label">First Name: </label>
                        <TextInput
                            className="form-control"
                            name="firstName"
                            value={firstName}
                            onChange={(e) => setFirstName(e.target.value)}
                        />
                    </div>

                    <div className="form-group">
                        <label className="control-label">Last Name: </label>
                        <TextInput
                            className="form-control"
                            name="lastName"
                            value={lastName}
                            onChange={(e) => setLastName(e.target.value)}
                        />
                    </div>

                    <div className="form-group">
                        <label className="control-label">Email: </label>
                        <TextInput
                            className="form-control"
                            name="email"
                            value={email}
                            onChange={(e) => setEmail(e.target.value)}
                        />
                    </div>

                    <div className="form-group">
                        <label className="control-label">User Name: </label>
                        <TextInput
                            className="form-control"
                            name="username"
                            value={username}
                            onChange={(e) => setUsername(e.target.value)}
                        />
                    </div>

                    <div className="form-group">
                        <label className="control-label">Role: </label>
                        <TextInput
                            className="form-control"
                            name="roles"
                            value={role}
                            onChange={(e) => setRole(e.target.value)}
                        />
                    </div>

                    <div className="form-group">
                        <label className="control-label">Password: </label>
                        <TextInput
                            className="form-control"
                            type="password"
                            name="password"
                            value={password}
                            onChange={(e) => setPassword(e.target.value)}
                        />
                    </div>

                    <div className="form-group">
                        <label className="control-label">Confirm Password: </label>
                        <TextInput
                            className="form-control"
                            type="password"
                            name="confirmationPassword"
                            value={confirmationPassword}
                            onChange={(e) => setConfirmationPassword(e.target.value)}
                        />
                    </div>
                    <br />
                    <div className="form-group">
                        <input type="submit" value="Create User" className="btn btn-primary"></input> &nbsp; &nbsp; 
                        <input type="button" value="Back" onClick={onClickBack} className="btn btn-primary"></input>
                    </div>
                </form>
            </div>
        </div>
    );
}