import { getData, putData, EndPoints } from "../services/AccessAPI";
import { useNavigate, useLocation } from "react-router-dom";
import { useEffect } from "react";
import { useState } from 'react';
import { Routes } from "../services/Routes";
import TextInput from "../ui-elements/controls/input/TextInput";

export default function UpdateUser() {
    let navigate = useNavigate();
    const search = useLocation().search;
    const id = new URLSearchParams(search).get('id')

    const [firstName, setFirstName] = useState("");
    const [lastName, setLastName] = useState("");
    const [email, setEmail] = useState("");
    const [username, setUsername] = useState("");
    const [role, setRole] = useState("");
    const [loading, setLoading] = useState(false);

    useEffect(() => {

        //const id = location.pathName.slice(location.pathName.lastIndexOf("/") , location.pathName.length);
        getData(EndPoints.USERS + "/" + id).then(
            (result) => {
                //let responseJson = result;
                console.log("user for edit: ");
                console.log(result);
                console.log("Username: " + result.username)
                if (result) {
                    setFirstName(result.firstName);
                    setLastName(result.lastName);
                    setEmail(result.email);
                    setUsername(result.username);
                    setRole(result.roles);
                    setLoading(false);
                }
            }
        );
    }, [id])

    function onKeyDown(e) {
        if (e.key === 'Enter') {
            this.update(false);
        }
    }
    
    function onUpdateCancel() {
        navigate(Routes.USERS);
    }

    function onSubmit(e){
        e.preventDefault();
        
        //const id = location.pathName.slice(location.pathName.lastIndexOf("/") , location.pathName.length);

        let userProfile = {
            id: id,
            Username: username,
            FirstName: firstName,
            LastName: lastName,
            Email: email,
        }

        putData(EndPoints.USERS + "/" + id, userProfile).then((result) => {
                console.log("Put Request body: ");
                console.log(userProfile);

                let responseJson = result;
                console.log("update response: ");
                
                if(responseJson){
                    console.log(responseJson);
                    navigate(Routes.USERS);
                }
            }
        );
    }

    return(
        <div className="row">
            <div className="col-md-4">
                <h3>Edit User</h3>
                <form onSubmit={onSubmit}>
                    <div className="form-group">
                        <label className="control-label">First Name: </label>
                        <TextInput
                            className="form-control"
                            value={firstName}
                            onChange={(e) => setFirstName(e.target.value)}
                            name="firstName"
                            onKeyDown={onKeyDown}
                        />
                    </div>

                    <br />
                    <div className="form-group">
                        <label className="control-label">Last Name: </label>
                        <TextInput
                            className="form-control"
                            value={lastName}
                            onChange={(e) => setLastName(e.target.value)}
                            name="lastName"
                            onKeyDown={onKeyDown}
                        />
                    </div>

                    <br />
                    <div className="form-group">
                        <label className="control-label">User Name: </label>
                        <TextInput
                            className="form-control"
                            value={username}
                            onChange={(e) => setUsername(e.target.value)}
                            name="username"
                            onKeyDown={onKeyDown}
                        />
                    </div>

                    <br />
                    <div className="form-group">
                        <label className="control-label">Email: </label>
                        <TextInput
                            className="form-control"
                            value={email}
                            onChange={(e) => setEmail(e.target.value)}
                            name="email"
                            onKeyDown={onKeyDown}
                        />
                    </div>

                    <br />
                    <div className="form-group">
                        <button onClick={onUpdateCancel} className="btn btn-primary">Cancel</button>
                        &nbsp;&nbsp;
                        <input type="submit" value="Edit" className="btn btn-warning"></input>
                    </div>
                </form>
            </div>
        </div>
    );
}