import { getData, EndPoints } from "../services/AccessAPI";
import { useNavigate } from "react-router-dom";
import { useEffect } from "react";
import { useState } from 'react';
import { Routes } from "../services/Routes";
import RowButton from "../ui-elements/table/RowButton";
import EditButton from "../ui-elements/controls/buttons/EditButton";
import DeleteButton from "../ui-elements/controls/buttons/DeleteButton";

export default function Users() {
    useEffect(() => {
        getAllUsersData();
    }, [])

    let navigate = useNavigate();

    const [state, setState] = useState({
        users: [],
        loading: false
    });

    function onUserCreate() {
        navigate(Routes.USERS_CREATE);
    }

    function onUserEdit(id){
        let query = "?id=" + id
        navigate(Routes.USERS_EDIT + query);
    }

    function onUserDelete(id){
        let query = "?id=" + id
        navigate(Routes.USERS_DELETE + query);
    }

    function getAllUsersData() {
        getData(EndPoints.USERS).then(
            (result) => {
                if (result) {
                    console.log(result);
                    setState({
                        users: result,
                        loading: false
                    });
                }
            }
        );
    }

    /*function renderAllUsersTable(users) {
        return (
            <table className="table table-striped">
                <thead>
                    <tr>
                        <th>First Name</th>
                        <th>Last Name</th>
                        <th>User Name</th>
                        <th>Email</th>
                        <th>Actions</th>
                    </tr>
                </thead>
                <tbody>
                    {
                        users.map(user => (
                            <tr key={user.id}>
                                <td>{user.firstName}</td>
                                <td>{user.lastName}</td>
                                <td>{user.username}</td>
                                <td>{user.email}</td>
                                <td><button onClick={() => onUserEdit(user.id)} className="btn btn-success">Edit</button> ||
                                    <button onClick={() => onUserDelete(user.id)} className="btn btn-danger">Delete</button></td>
                            </tr>
                        ))
                    }
                </tbody>
            </table>
        );
    }*/

    function renderBootstrapTable(users) {
        return (
            <div className="container-fluid">
                <div className="row">
                    <div className="col-md-12">
                        <table className="table table-sm table-striped">
                            <thead>
                                <tr>
                                    <th>First Name</th>
                                    <th>Last Name</th>
                                    <th>User Name</th>
                                    <th>Email</th>
                                    <th>Actions</th>
                                </tr>
                            </thead>
                            <tbody>
                                {
                                    users.map(user => (
                                        <tr key={user.id}>
                                            <td>{user.firstName}</td>
                                            <td>{user.lastName}</td>
                                            <td>{user.username}</td>
                                            <td>{user.email}</td>
                                            <td>
                                                <EditButton
                                                    className="btn btn-sm btn-primary"
                                                    onClick={() => onUserEdit(user.id)}
                                                />
                                                &nbsp;
                                                <DeleteButton
                                                    className="btn btn-sm btn-danger"
                                                    onClick={() => onUserDelete(user.id)}
                                                />
                                            </td>
                                        </tr>
                                    ))
                                }
                            </tbody>
                        </table>
                    </div>
                </div>
            </div>
        );
    }

    let content = state.loading ? (
        <p>
            <em>Loading...</em>
        </p>
    ) : (
        //renderAllUsersTable(state.users)
        renderBootstrapTable(state.users)
    )

    
    return (
        <div>
            <h3>List of Users</h3>
            <button onClick={() => onUserCreate()} className="btn btn-primary">Create new user</button>
            {content}
        </div>
    );
}