namespace ActiveAllocator.API.Controllers;

using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using ActiveAllocator.API.Models.CurrencyType;
using ActiveAllocator.API.Services;
using ActiveAllocator.API.Config;
using Microsoft.AspNetCore.Authorization;

[Authorize]
[ApiController]
[Route("[controller]")]
public class CurrencyTypesController : ControllerBase
{
    private ICurrencyTypeService _currencyTypeService;
    private IMapper _mapper;
    private readonly AppSettings _appSettings;

    public CurrencyTypesController(
        ICurrencyTypeService currencyTypeService,
        IMapper mapper,
        IOptions<AppSettings> appSettings)
    {
        _currencyTypeService = currencyTypeService;
        _mapper = mapper;
        _appSettings = appSettings.Value;
    }

    [HttpGet]
    public IActionResult GetAll()
    {
        var currencyTypes = _currencyTypeService.GetAll();
        return Ok(currencyTypes);
    }

    [HttpGet("{id}")]
    public IActionResult GetById(int id)
    {
        var currencyType = _currencyTypeService.GetById(id);
        return Ok(currencyType);
    }

    [HttpPost]
    public IActionResult Create([FromBody]CurrencyTypeCreateRequest model)
    {
        _currencyTypeService.Create(model);
        return Ok(new { message = "currencyType created" });
    }

    /*[HttpPut("{id}")]
    public IActionResult Update(int id, [FromBody]AccountUpdateRequest model)
    {
        _accountService.Update(id, model);
        return Ok(new { message = "account updated" });
    }*/

    [HttpDelete("{id}")]
    public IActionResult Delete(int id)
    {
        _currencyTypeService.Delete(id);
        return Ok(new { message = "currencyType deleted" });
    }
}