namespace ActiveAllocator.API.Controllers;

using AutoMapper;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using ActiveAllocator.API.Models.Envelopes;
using ActiveAllocator.API.Services;
using ActiveAllocator.API.Config;
using System.Collections.Generic;
using ActiveAllocator.API.Entities;
using Microsoft.AspNetCore.Authorization;

[Authorize]
[ApiController]
[Route("[controller]")]
public class EnvelopesController : ControllerBase
{
    private IEnvelopeService _envelopeService;
    private IMapper _mapper;
    private readonly AppSettings _appSettings;

    public EnvelopesController(
        IEnvelopeService envelopeService,
        IMapper mapper,
        IOptions<AppSettings> appSettings)
    {
        _envelopeService = envelopeService;
        _mapper = mapper;
        _appSettings = appSettings.Value;
    }

    [HttpGet]
    public IActionResult GetAll(int? accountId = null)
    {
        List<EnvelopeDTO> envelopeDtos = new List<EnvelopeDTO>();

        foreach (Envelope env in _envelopeService.GetAll(accountId))
        {
            List<EnvelopeFundingMethodDTO> methodDtos = new List<EnvelopeFundingMethodDTO>();

            foreach (EnvelopeFundingMethod method in env.FundingMethods)
            {
                methodDtos.Add(_mapper.Map<EnvelopeFundingMethod, EnvelopeFundingMethodDTO>(method));
            }

            EnvelopeDTO dto = new EnvelopeDTO {
                Id = env.Id,
                Name = env.Name,
                AccountId = env.Account == null ? 0 : env.Account.Id,
                Enabled = env.Enabled,
                Period = env.Period,
                IsPersistant = env.IsPersistant,
                Priority = env.Priority,
                Balance = env.Balance,
                InitialBalance = env.InitialBalance,
                LastTriggeredOn = env.LastTriggeredOn,
                FundingMethods = methodDtos
            };

            envelopeDtos.Add(dto);
        }
            //envelopeDtos.Add(_mapper.Map<Envelope, EnvelopeDTO>(env));

        return Ok(envelopeDtos);
    }

    [HttpGet("{id}")]
    public IActionResult GetById(int id)
    {
        Envelope envelope = _envelopeService.GetById(id);
        return Ok(_mapper.Map<Envelope, EnvelopeDTO>(envelope));
    }

    [HttpPost]
    public IActionResult Create([FromBody]EnvelopeCreateRequest model)
    {
        _envelopeService.Create(model);
        return Ok(new { message = "envelope created" });
    }

    [HttpPut("{id}")]
    public IActionResult Update(int id, [FromBody]EnvelopeUpdateRequest model)
    {
        _envelopeService.Update(id, model);
        return Ok(new { message = "envelope updated" });
    }

    [HttpDelete("{id}")]
    public IActionResult Delete(int id)
    {
        _envelopeService.Delete(id);
        return Ok(new { message = "envelope deleted" });
    }

    [HttpGet("Historical/{id}")]
    public IActionResult Historical(int id, int lookBack)
    {
        EnvelopeHistorical hist = _envelopeService.GetEnvelopeHistorical(id, lookBack);
        return Ok(_mapper.Map<EnvelopeHistorical, EnvelopeHistoricalDTO>(hist));
    }

    [HttpGet("FundingMethodModes")]
    public IActionResult FundingMethodModes()
    {
        List<EnvelopeFundingMethodModes> modes = _envelopeService.GetEnvelopeFundingMethodModes();
        return Ok(modes);
    }
}