using System.Collections;
using System.Collections.Generic;
using System.Text.Json.Serialization;
using ActiveAllocator.API.Entities;

namespace ActiveAllocator.API.Models.Autoclass;

public class AutoclassFieldMetaDTO
{
    public AutoclassFieldMetaDTO(AutoclassTransactionField name)
    {
        Name = name;
    }

    public AutoclassTransactionField Name { get; set; }
    public string Type {
        get
        {
            switch (this.Name)
            {
                case AutoclassTransactionField.AMOUNT:
                    return "DECIMAL";
                case AutoclassTransactionField.DATE:
                    return "DATETIME";
                case AutoclassTransactionField.DESCRIPTION:
                case AutoclassTransactionField.EXTERNAL_ID:
                    return "STRING";
                case AutoclassTransactionField.IS_PENDING:
                    return "BOOLEAN";
                case AutoclassTransactionField.CURRENCY_TYPE:
                    return "CURRENCYTYPE";
                case AutoclassTransactionField.CREDIT_ACCOUNT:
                case AutoclassTransactionField.DEBIT_ACCOUNT:
                    return "ACCOUNT";
                case AutoclassTransactionField.CREDIT_ENVELOPE:
                case AutoclassTransactionField.DEBIT_ENVELOPE:
                    return "ENVELOPE";
                case AutoclassTransactionField.TAGS:
                    return "STRING_ARRAY";
                default:
                    return "ERROR";
            }
        }
    }
}


public class AutoclassTypeMetaDTO
{
    public AutoclassType Type { get; set; }
    public List<AutoclassOperatorMetaDTO> Operators { get; set; }

    public AutoclassTypeMetaDTO(AutoclassType type)
    {
        Type = type;

        switch(type)
        {
            case AutoclassType.DATETIME:
                Operators = new List<AutoclassOperatorMetaDTO>() {
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.EQUAL),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.NOT_EQUAL),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.GREATER_THAN),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.GREATER_THAN_OR_EQUAL),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.LESS_THAN),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.LESS_THAN_OR_EQUAL),
                };
                break;
            case AutoclassType.STRING:
                Operators = new List<AutoclassOperatorMetaDTO>() {
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.EQUAL),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.NOT_EQUAL),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.CONTAINS),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.CONTAINS_INSENSITIVE),
                };
                break;
            case AutoclassType.DECIMAL:
                Operators = new List<AutoclassOperatorMetaDTO>() {
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.EQUAL),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.NOT_EQUAL),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.GREATER_THAN),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.GREATER_THAN_OR_EQUAL),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.LESS_THAN),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.LESS_THAN_OR_EQUAL),
                };
                break;
            case AutoclassType.BOOLEAN:
                Operators = new List<AutoclassOperatorMetaDTO>() {
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.EQUAL),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.NOT_EQUAL),
                };
                break;
            case AutoclassType.ACCOUNT:
                Operators = new List<AutoclassOperatorMetaDTO>() {
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.EQUAL),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.NOT_EQUAL),
                };
                break;
            case AutoclassType.ENVELOPE:
                Operators = new List<AutoclassOperatorMetaDTO>() {
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.EQUAL),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.NOT_EQUAL),
                };
                break;
            case AutoclassType.CURRENCYTYPE:
                Operators = new List<AutoclassOperatorMetaDTO>() {
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.EQUAL),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.NOT_EQUAL),
                };
                break;
            case AutoclassType.STRING_ARRAY:
                Operators = new List<AutoclassOperatorMetaDTO>() {
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.CONTAINS),
                    new AutoclassOperatorMetaDTO(AutoclassCompareOperator.CONTAINS_INSENSITIVE),
                };
                break;
        }
    }
}

public class AutoclassOperatorMetaDTO
{
    public AutoclassOperatorMetaDTO(AutoclassCompareOperator op)
    {
        Operator = op;
    }

    public AutoclassCompareOperator Operator { get; set; }
    public string DisplayName {
        get
        {
            switch (this.Operator)
            {
                case AutoclassCompareOperator.EQUAL:
                    return "=";
                case AutoclassCompareOperator.NOT_EQUAL:
                    return "!=";
                case AutoclassCompareOperator.GREATER_THAN:
                    return ">";
                case AutoclassCompareOperator.GREATER_THAN_OR_EQUAL:
                    return ">=";
                case AutoclassCompareOperator.LESS_THAN:
                    return "<";
                case AutoclassCompareOperator.LESS_THAN_OR_EQUAL:
                    return "<=";
                case AutoclassCompareOperator.CONTAINS:
                    return "Contains";
                case AutoclassCompareOperator.CONTAINS_INSENSITIVE:
                    return "Contains (case insensitive)";
                default:
                    return "ERROR";
            }
        }
    }
}