using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using ActiveAllocator.API.Configs;
using ActiveAllocator.API.Entities;
using ActiveAllocator.API.Models.Envelopes;
using ActiveAllocator.API.Models.Transactions;
using ActiveAllocator.API.Services;
using Internal;
using Microsoft.Extensions.Options;

namespace ActiveAllocator.API.Processes;

public class EnvelopeFundingProcess : IEnvelopeFundingProcess
{
   private readonly EnvelopeFundConfig _config;
   private readonly ILogger<EnvelopeFundingProcess> _logger;

   public EnvelopeFundingProcess(
        IOptions<EnvelopeFundConfig> config,
        ILogger<EnvelopeFundingProcess> logger)
   {
      _config = config.Value;
      _logger = logger;
   }

   public async Task FundEnvelopes(IEnvelopeService envelopeService)
   {
        _logger.LogInformation($"Starting envelope funding...");
         
        // Grab envelopes
        List<Envelope> envelopes = envelopeService.GetAll().ToList();

        bool anyTriggered = false;
        
        foreach (Envelope env in envelopes)
        {
            // Check if needs triggering
            if (env.Enabled && !envelopeService.HasBeenFundedThisMonth(env.Id))
            {
                _logger.LogInformation($"Envelope '{env.Name}' (id={env.Id}) needs funding - Has now been triggered.");
                anyTriggered = true;

                // Fund Envelope
                envelopeService.FundEnvelopeForThisMonth(env.Id);
            }
        }

        if (!anyTriggered)
            _logger.LogInformation($"No action taken.");
        else
            _logger.LogInformation($"Envelope funding complete.");
   }
}
