namespace ActiveAllocator.API.Services;

using AutoMapper;
using BCrypt.Net;
using ActiveAllocator.API.Authorization;
using ActiveAllocator.API.Config;
using System;
using System.Security.Cryptography;

public interface IApiKeyService
{
    string GenerateApiKey();
}

public class ApiKeyService : IApiKeyService
{
    private DataContext _context;
    private IJwtUtils _jwtUtils;
    private readonly IMapper _mapper;

    public ApiKeyService(
        DataContext context,
        IMapper mapper,
        IJwtUtils jwtUtils)
    {
        _context = context;
        _mapper = mapper;
        _jwtUtils = jwtUtils;
    }

    private const string _prefix = "CT-";
    private const int _numberOfSecureBytesToGenerate = 32;
    private const int _lengthOfKey = 32;
    
    public string GenerateApiKey()
    {
        var bytes = RandomNumberGenerator.GetBytes(_numberOfSecureBytesToGenerate);

        string base64String = Convert.ToBase64String(bytes)
            .Replace("+", "-")
            .Replace("/", "_");
        
        var keyLength = _lengthOfKey - _prefix.Length; 

        return _prefix + base64String[..keyLength];
    }
}