namespace ActiveAllocator.API.Services;

using AutoMapper;
using BCrypt.Net;
using ActiveAllocator.API.Entities;
using ActiveAllocator.API.Config;
using ActiveAllocator.API.Models.CurrencyType;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

public interface ICurrencyTypeService
{
    IEnumerable<CurrencyType> GetAll();
    CurrencyType GetById(int id);
    void Create(CurrencyTypeCreateRequest model);
    //void Update(int id, AccountUpdateRequest model);
    void Delete(int id);
}

public class CurrencyTypeService : ICurrencyTypeService
{
    private DataContext _context;
    private readonly IMapper _mapper;

    public CurrencyTypeService(
        DataContext context,
        IMapper mapper)
    {
        _context = context;
        _mapper = mapper;
        
        if (_context.CurrencyTypes.Where(ct => ct.Code == "USD").ToList().Count() < 1)
            this.createUSDCurrencyType();
    }

    private void createUSDCurrencyType()
    {
        _context.CurrencyTypes.Add(new CurrencyType() {
            Code = "USD",
            Symbol = "$",
            DecimalPlaces = 2
        });

        _context.SaveChanges();
    }

    public IEnumerable<CurrencyType> GetAll()
    {
        return _context.CurrencyTypes;
    }

    public CurrencyType GetById(int id)
    {
        return getCurrencyType(id);
    }

    public void Create(CurrencyTypeCreateRequest model)
    {
        // Check that CurrencyType with same code doesn't exist
        IEnumerable<CurrencyType> sameCode = _context.CurrencyTypes.Where(a => a.Code.ToUpper() == model.Code.ToUpper());

        if (sameCode.Count() > 0)
            throw new AppException("CurrencyType with code '" + model.Code + "' already exists");

        // Check that CurrencyType with same symbol doesn't exist
        IEnumerable<CurrencyType> sameSymbol = _context.CurrencyTypes.Where(a => a.Symbol.ToUpper() == model.Symbol.ToUpper());

        if (sameSymbol.Count() > 0)
            throw new AppException("CurrencyType with symbol '" + model.Symbol + "' already exists");

        // map model to new account object
        var currencyType = _mapper.Map<CurrencyType>(model);

        _context.CurrencyTypes.Add(currencyType);
        _context.SaveChanges();
    }

    /*public void Update(int id, AccountUpdateRequest model)
    {
        var account = getAccount(id);

        // validate
        if (model.Name != account.Name && _context.Accounts.Any(x => x.Name == model.Name))
            throw new AppException("Account with the name '" + model.Name + "' already exists");

        // copy model to account and save
        _mapper.Map(model, account);
        _context.Accounts.Update(account);
        _context.SaveChanges();
    }*/

    public void Delete(int id)
    {
        var currencyType = getCurrencyType(id);
        _context.CurrencyTypes.Remove(currencyType);
        _context.SaveChanges();
    }

    // helper methods

    private CurrencyType getCurrencyType(int id)
    {
        var currencyType = _context.CurrencyTypes.Find(id);
        if (currencyType == null) throw new KeyNotFoundException("CurrencyType not found");
        return currencyType;
    }
}